/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.client.internal.CompositeWebClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("jazz.app.context");

(function() { //module container

var ServiceRequest          = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient        = com.ibm.team.repository.web.transport.TeamServerClient;


// The composite web service attempts to reduce traffic between the client and the server by providing services
// tailored to the need of the client.  Often, multiple underlying services are called within the server, and a composite
// response is returned to the client. 
com.ibm.asq.common.web.client.internal.CompositeWebClient = {

    PROCESS_SERVICE_URI: "com.ibm.rqm.web.common.service.rest.ICompositeWebRestService",
	TYPE_TESTPLAN: "TestPlan",
	TYPE_TESTCASE: "TestCase",
	TYPE_TESTSUITE: "TestSuite",
	TYPE_TESTSCRIPT: "TestScript",
	TYPE_TESTDATA: "TestData",
	TYPE_KEYWORD: "Keyword",
	TYPE_TCER: "TestcaseExecutionRecord",
	TYPE_TSER: "TestSuiteExecutionRecord",
	TYPE_TCRESULT: "ExecutionResult",
	TYPE_TSRESULT: "ExecutionResultGroup",
	TYPE_EXESCHEDULE: "ExecutionSchedule",
	TYPE_EXECONSOLE: "ExecutionConsole",
	TYPE_ADAPTERCONSOLE: "AdapterConsole",
	TYPE_SCHEDULEDJOB: "ScheduledJobs",
	TYPE_BUILDRECORD: "BuildRecord",
	TYPE_BUILDDEFINITION: "BuildDefinition",
    
    _invokeService: function(responseHandler, method, args){
        var serviceRequest = new ServiceRequest(this.PROCESS_SERVICE_URI, method, args);
        TeamServerClient.invokeService(serviceRequest, responseHandler);
    },
    // collects all the information needed when opening an artifact.  All of the TYPE_* artifacts above are supported.
    getBrowseInitInfo: function(serviceResponseHandler, processAreaItemId, artifactType ) {
    	var args = {
    			'processArea': processAreaItemId, 
    			'artifactType': artifactType
    	};
    	if (arguments.length == 4) {
    		args.providerTypeId = arguments[3];
    	}
    	return this._invokeService(serviceResponseHandler, "getBrowseInitData", args);
    },
    
    // collects all the information needed when opening a page to create an artifact.  Note that this does not include the actual artifact creation.
    // Only these aretifacts are supported: TestPlan, TestCase, TestSuite, TestScript, TestData, TestcaseExecutionRecord, TestSuiteExecutionRecord,
	// ExecutionSchedule, BuildRecord or BuildDefinition
    // id is the artifact id if we are opening a specific artifact.  Support for this is not yet complete, we may also support itemId.
    getOpenCreateInfo: function(serviceResponseHandler, processAreaItemId, artifactType, providerTypeId, id, isRefresh, isNew ) {
		var jsonHandler = this._getJSONHandler(serviceResponseHandler);

    	var args = {
    			'processArea': processAreaItemId, 
    			'artifactType': artifactType
    	};
    	if (providerTypeId) {
    		args.providerTypeId = providerTypeId;
    	}
    	if (id) {
    		args.id = id;
    	}
    	if(isRefresh){
    		args.isRefresh = true;
    	}
    	if(isNew){
    		args.isNew = isNew;
    	}
    	return this._invokeService(jsonHandler, "getOpenCreateData", args);
    },
    
    getArtifactInfo: function(serviceResponseHandler, processArea, artifactType, id, itemId, revision){
    	var jsonHandler = this._getJSONHandler(serviceResponseHandler);
    	var args = {
    		'processArea': processArea, 
    		'artifactType': artifactType
    	};    	
    	if (id) {
    		args.id = id;
    	}
    	if(itemId){
    		args.itemId = itemId;
    	} 
    	if(revision){
    		args.revision = revision;
    	}
    	return this._invokeService(jsonHandler, "getArtifact", args);
    },
    
    postPrepareExecution: function(responseHandler, args) {
    	args = dojo.mixin(args, {
    		projectAreaItemId: jazz.app.context.get().itemId
    	});
    	return this._invokeService(responseHandler, "postPrepareExecution", args);
    },
    
    postStartExecution: function(responseHandler, args) {
    	args = dojo.mixin(args, {
    		projectAreaItemId: jazz.app.context.get().itemId
    	});
    	return this._invokeService(responseHandler, "postStartExecution", args);
    },
    postExecutionStep: function(responseHandler, args){
    	return this._invokeService(responseHandler, "postExecutionStep", args);
    },
    
	/*
	 * Wrap the ServiceResponseHandler to automatically parse the JSON string from 
	 * some server calls and provide the JSON object to the original handler  
	 */
    _getJSONHandler: function(originalHandler){
        var jsonHandlerObj = {
            success: function(/*String */ compositeDTO){
            	var permissions = compositeDTO.checkPermission;
            	if (permissions) {
                	compositeDTO.checkPermission = dojo.fromJson(permissions);
            	}
                //wrap the JSON transformation                
                originalHandler.doCallback(compositeDTO);
            },
            failure: function(error){
                originalHandler.doErrorCallback(error);
            }
        };
        return new ServiceResponseHandler(jsonHandlerObj, "success", "failure");
    }	

};

})();
