<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    exclude-result-prefixes="xsl HttpClientExtension log">
    
    <!-- templates -->
    <xsl:template match="table-list/rqm:suiteelement">
        <xsl:param name="columns"/>
        
        <xsl:apply-templates mode="se-copy" select=".">
            <xsl:with-param name="columns" select="$columns"/>
        </xsl:apply-templates>
    </xsl:template>
    
    <xsl:template mode="se-copy" match="@*|node()">
        <xsl:param name="columns"/>
        
        <xsl:copy>
            <xsl:apply-templates mode="se-copy" select="@*|node()">
                <xsl:with-param name="columns" select="$columns"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="se-copy" match="rqm:executionscript[@href]">
    	<xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/executionscript/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="se-copy" match="rqm:configuration[@href]">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/configuration/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="se-copy" match="rqm:testcase[@href]">
        <xsl:param name="columns"/>
        
        <xsl:variable name="fields">
            <xsl:variable name="tmp-fields">
                <xsl:text>identifier|</xsl:text>
                <xsl:for-each select="$columns">
                    <xsl:apply-templates mode="se-param" select="pref_table:name"/>
                </xsl:for-each>
            </xsl:variable>
            
            <xsl:value-of select="substring($tmp-fields, 0, string-length($tmp-fields))"/>
        </xsl:variable>
        
        <xsl:if test="log:isDebugEnabled()">
            <xsl:value-of select="log:debug(concat('testcase $fields=', $fields))"/></xsl:if>
        
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, concat('fields=feed/entry/content/testcase/(', $fields, ')'))"/>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name">
        <!-- intentionally left blank -->
        <!-- ignores all columns that are not otherwise matched by a template -->
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='Execution Order']">
        <!-- TODO: figure out where this comes from -->
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='ID']">
        <xsl:text>webId|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='Priority']">
        <xsl:text>priority|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='Test Case']">
        <xsl:text>title|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='State']">
        <xsl:text>state|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='Estimate']">
        <xsl:text>estimate|</xsl:text>
    </xsl:template>
    
    <xsl:template mode="se-param" match="pref_table:column/pref_table:name[.='Modified']">
        <xsl:text>updated|</xsl:text>
    </xsl:template>
    
</xsl:stylesheet>