<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:atom="http://www.w3.org/2005/Atom"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:vega="http://schema.ibm.com/vega/2008/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestplanProxy"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    exclude-result-prefixes="xsl xalan atom rqm dc alm vega prep exslt HttpClientExtension proxy utils log">
    
    <!-- included stylesheets -->
    <xsl:import  href="cache://stylesheets/prep/abstract-prep-handler.xsl"/>
    <xsl:include href="cache://stylesheets/_util/external-requirement-href.xsl"/>
    
    <!-- params -->
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="integrationService"/>
    <xsl:param name="processService"/>
    <xsl:param name="testPhaseService"/>
    <xsl:param name="testPhaseRestService"/>
    <xsl:param name="catTypeRestService"/>
    
    <!-- proxy definition -->
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestplanProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $integrationService, $processService, $testPhaseService, $testPhaseRestService, $catTypeRestService)"/>
    
    <!-- keys -->
    <xsl:key name="objectives-by-group" match="rqm:objectiveStatus" use="rqm:group"/>
    
    <!-- templates -->
    <xsl:template match="rqm:testplan">
        <xsl:copy>
            <xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:description"/>
            <xsl:apply-templates select="alm:owner"/>
            <xsl:apply-templates select="dc:creator"/>
            <xsl:apply-templates select="alm:updated"/>
            <xsl:apply-templates select="rqm:creationDate"/>
            <xsl:apply-templates select="alm:state"/>
            <xsl:apply-templates select="rqm:priority"/>
            <xsl:apply-templates select="rqm:copiedFrom"/>
            <xsl:call-template name="_insert-copiedTo"/>
            
            <xsl:if test="log:isDebugEnabled()">
                <xsl:value-of select="log:debug_xml(proxy:getSections($item))"/></xsl:if>
            
            <xsl:apply-templates select="proxy:getSections($item)/rqm:section">
                <xsl:with-param name="testplan" select="current()"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="rqm:snapshot"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- copiedFrom -->
    <xsl:template match="rqm:copiedFrom">
        <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=/feed/entry/content/testplan/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:testplan">
        <prep:copiedFrom>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <!-- copiedTo -->
    <xsl:template name="_insert-copiedTo">
        <xsl:variable name="copiedToFeed">
            <xsl:variable name="tpFilter">copiedFrom/@href="<xsl:value-of select="utils:encodeUri(dc:identifier)"/>"</xsl:variable>

            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../testplan'"/>
                <xsl:with-param name="query" select="concat('fields=/feed/entry/content/testplan[', $tpFilter, ']/(identifier|title)')"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:apply-templates mode="copiedTo" select="exslt:node-set($copiedToFeed)//rqm:testplan"/>
    </xsl:template>
    
    <xsl:template mode="copiedTo" match="rqm:testplan">
        <prep:copiedTo xsl:exclude-result-prefixes="vega rqm">
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedTo>
    </xsl:template>
    
    <!-- Summary -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planSummary']">
        <xsl:param name="testplan"/>
        <xsl:variable name="categories" select="proxy:getCategories($item)/rqm:category"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $categories or $testplan/rqm:customAttributes">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$categories"/>
                <xsl:apply-templates select="$testplan/rqm:customAttributes"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Business Objectives -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Objectives -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Strategy -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Formal Review -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.editor.section.review']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:approvals">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$testplan/rqm:approvals"/>    
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Requirement Collection Links -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.defects.web.RmColLinks']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:requirementset or $testplan/rqm:requirementView or $testplan/rqm:requirementPackage">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$testplan/rqm:requirementset">
                    <prep:requirementSets>
                        <xsl:apply-templates select="$testplan/rqm:requirementset"/>
                    </prep:requirementSets>    
                </xsl:if>
                <xsl:if test="$testplan/rqm:requirementView">
                    <prep:requirementViews>
                        <xsl:apply-templates select="$testplan/rqm:requirementView"/>
                    </prep:requirementViews>    
                </xsl:if>
                <xsl:if test="$testplan/rqm:requirementPackage">
                    <prep:requirementPackages>
                        <xsl:apply-templates select="$testplan/rqm:requirementPackage"/>
                    </prep:requirementPackages>    
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Requirements -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.requirements.editor.section.planRequirements']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:requirementset or $testplan/rqm:requirement">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            
            <prep:requirements>
            	<xsl:if test="$testplan/rqm:requirementset">
            		<xsl:apply-templates select="$testplan/rqm:requirementset"/>
            	</xsl:if>
                <xsl:if test="$testplan/rqm:requirement">  <!-- Check for Requisite Pro Requirements -->
                	<xsl:apply-templates select="$testplan/rqm:requirement"/>
                </xsl:if>
            </prep:requirements>
        </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Development Plan Links -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.defects.web.CmDevPlanLinks']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:developmentplan">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$testplan/rqm:developmentplan">
                    <prep:developmentPlans>
                        <xsl:apply-templates select="$testplan/rqm:developmentplan"/>    
                    </prep:developmentPlans>    
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Schedule -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.editor.section.timebox']">
        <xsl:param name="testplan"/>
        
        <xsl:variable name="testPhaseSet" select="proxy:getTestPhases($item)"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testPhaseSet or $testplan/rqm:keydates">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$testPhaseSet">
                    <table-list>
                        <xsl:attribute name="key">RQM-ITERATIONS-VIEWER</xsl:attribute>
                        <xsl:apply-templates select="$testPhaseSet"/>
                    </table-list>    
                </xsl:if>
                
                <xsl:if test="$testplan/rqm:keydates">
                    <xsl:apply-templates select="$testplan/rqm:keydates"/>    
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Estimation -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.editor.section.sizing']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:estimation">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$testplan/rqm:estimation/rqm:planningEffortPersonHours != 0.0">
                    <rqm:planningEffort><xsl:value-of select="$testplan/rqm:estimation/rqm:planningEffortPersonHours"/></rqm:planningEffort>
                </xsl:if>
                
                <xsl:if test="$testplan/rqm:estimation/rqm:executionEffortPersonHours != 0.0">
                    <rqm:executionEffort><xsl:value-of select="$testplan/rqm:estimation/rqm:executionEffortPersonHours"/></rqm:executionEffort>
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Configuration -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planEnvironment']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:configuration">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$testplan/rqm:configuration">
                    <prep:configurations>
                        <xsl:for-each select="$testplan/rqm:configuration">
                            <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/configuration/(identifier|name|configuration)')"/>
                        </xsl:for-each>    
                    </prep:configurations>
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Team Area -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestTeam']">
        <xsl:param name="testplan"/>
        <xsl:variable name="href" select="$testplan/rqm:legacyTeamArea/@href"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $href">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <xsl:if test="$href">
                    <xsl:apply-templates select="HttpClientExtension:get($restClient, $href)"/>        
                </xsl:if>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:contributor">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href)"/>
    </xsl:template>
    
    <!-- Quality Objectives -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.web.qualityGoalSection']">
        <xsl:param name="testplan"/>
        
        <xsl:call-template name="_buildObjectivesSection">
            <xsl:with-param name="objective-list" select="$testplan/rqm:objectives/rqm:objectiveStatus[rqm:group='com.ibm.rqm.process.web.qualityGoalSection']"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Entry Criteria -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.web.entryCriteria']">
        <xsl:param name="testplan"/>
        
        <xsl:call-template name="_buildObjectivesSection">
            <xsl:with-param name="objective-list" select="$testplan/rqm:objectives/rqm:objectiveStatus[rqm:group='com.ibm.rqm.process.web.entryCriteria']"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Exit Criteria -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.web.exitCriteria']">
        <xsl:param name="testplan"/>
        
        <xsl:call-template name="_buildObjectivesSection">
            <xsl:with-param name="objective-list" select="$testplan/rqm:objectives/rqm:objectiveStatus[rqm:group='com.ibm.rqm.process.web.exitCriteria']"/>
        </xsl:call-template>
    </xsl:template>
    
    <xsl:template name="_buildObjectivesSection">
        <xsl:param name="objective-list"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $objective-list">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$objective-list"/>    
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:objectiveStatus">
        <prep:objectiveStatus>
            <rqm:actualValue><xsl:value-of select="rqm:actualValue"/></rqm:actualValue>
            <rqm:state><xsl:value-of select="rqm:state"/></rqm:state>
            <rqm:comment><xsl:value-of select="rqm:comment"/></rqm:comment>
            <xsl:apply-templates select="HttpClientExtension:get($restClient, rqm:objective/@href)"/>
        </prep:objectiveStatus>
    </xsl:template>
    
    <xsl:template match="rqm:objective">
        <prep:objective>
            <dc:title><xsl:value-of select="dc:title"/></dc:title>
            <rqm:operator><xsl:value-of select="rqm:operator"/></rqm:operator>
            <rqm:targetValue><xsl:value-of select="rqm:targetValue"/></rqm:targetValue>
        </prep:objective>
    </xsl:template>
    
    <!-- Child Test Plans -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planChildren']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:childplan">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>

                <table-list>
                    <xsl:attribute name="key">RQM-TP-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates select="$testplan/rqm:childplan"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:childplan">
        <!-- There are two tricks going on here.  First, we change the name of the element from "childplan" to 
             "testplan" so that we can reuse the same templates in expand-tables.xsl that we use for the "View 
             all Test Plans" table.  Secondly, we replace the @vega:id that we were given with our own.  This is 
             because the one we are given is for an ITestPlan, and the code expects an IVersionedTestPlan. -->
        <rqm:testplan vega:id="{utils:getUuidByHref(@href)}">
            <xsl:apply-templates select="@href"/>
        </rqm:testplan>
    </xsl:template>
    
    <!-- Test Suites -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestSuites']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:testsuite">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list parent="{$testplan/dc:identifier}">
                    <xsl:attribute name="key">RQM-TSUITE-TABLEVIWER</xsl:attribute>
                    <xsl:apply-templates select="$testplan/rqm:testsuite"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:testsuite">
        <xsl:copy>
            <xsl:attribute name="uuid"><xsl:value-of select="@vega:id"/></xsl:attribute>
            <xsl:apply-templates select="@href"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Test Cases -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestCases']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:testcase">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list parent="{$testplan/dc:identifier}">
                    <xsl:attribute name="key">RQM-TC-TABLEVIWER</xsl:attribute>
                    <xsl:apply-templates select="$testplan/rqm:testcase"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Resources -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planResources']">
        <!-- TODO: not currently supported -->
    </xsl:template>
    
    <!-- Application Security -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.requirements.editor.section.planSecurity']">
        <!-- TODO: not currently supported -->
    </xsl:template>
    
    <!-- Risk Assessment -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.risk.editor.section.planAssessment']">
        <xsl:param name="testplan"/>
        <xsl:variable name="risk-text" select="$testplan/*[@extensionDisplayName='RQM-KEY-TP-RISK-TITLE']"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $risk-text or rqm:risk">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$risk-text"/>
                <xsl:apply-templates select="$testplan/rqm:risk"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Instruments and Test Equipment -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Attachments -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.attachments']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:attachment">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <prep:attachments>
                    <xsl:for-each select="$testplan/rqm:attachment">
                        <xsl:copy-of select="proxy:attachment($item, @href)"/>
                    </xsl:for-each>    
                </prep:attachments>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Data -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Bed -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Identification -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Plan Scope -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Normative and Informative Documents -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planNormativeInformative']">
        <xsl:param name="testplan"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testplan/rqm:com.ibm.rqm.planning.editor.section.planNormativeInformative">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()"/>
                <xsl:copy-of select="proxy:normativeAndInformativeJsonToXML($item, $testplan/rqm:com.ibm.rqm.planning.editor.section.planNormativeInformative)"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
    
    <!-- Test Conditions -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Sampling Strategy -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Software Test Environment Details -->
    <!-- Is a rich text section, and therefore, is handled by the Custom Section template -->
    
    <!-- Test Case Execution Records-->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestCaseExecutionRecords']">
        <xsl:param name="testplan"/>
        
        <xsl:variable name="ewis">
            <!-- collect all the EWIs for this test plan via an additional REST API query -->
            <xsl:variable name="testcaseQuery">
                <xsl:variable name="tmp">
                    <!-- build list of test cases that we need EWIs for -->
                    <xsl:for-each select="$testplan/rqm:testcase">
                        <xsl:text>testcase/@href='</xsl:text><xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, @href))"/><xsl:text>' or </xsl:text>
                    </xsl:for-each>    
                </xsl:variable>
                
                <xsl:if test="$tmp">
                    <!-- strip off the last ' or ' -->
                    <xsl:value-of select="substring($tmp, 0, string-length($tmp)-3)"/>
                </xsl:if>
            </xsl:variable>
            
            <xsl:if test="string-length($testcaseQuery) > 0">
                <xsl:call-template name="restGET">
                    <xsl:with-param name="path"  select="'../executionworkitem'"/>
                    <xsl:with-param name="query" select="concat('fields=feed/entry/content/executionworkitem[', $testcaseQuery, ']/(identifier|testplan)')"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:variable>
       
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $ewis">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>                
                <table-list>
                    <xsl:attribute name="key">RQM-TP-TCER-SECTION-CONTAINER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($ewis)/atom:feed/atom:entry/atom:content/rqm:executionworkitem[rqm:testplan/@href=$testplan/dc:identifier]"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Suite Execution Records -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestSuiteExecutionRecords']">
        <xsl:param name="testplan"/>
        
        <xsl:variable name="tsers">
            <!-- collect all the TSERs for this test plan via an additional REST API query -->
            <xsl:variable name="testsuiteQuery">
                <xsl:variable name="tmp">
                    <!-- build list of test suites that we need TSERs for -->
                    <xsl:for-each select="$testplan/rqm:testsuite">
                        <xsl:text>testsuite/@href='</xsl:text><xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, @href))"/><xsl:text>' or </xsl:text>
                    </xsl:for-each>    
                </xsl:variable>
                
                <xsl:if test="$tmp">
                    <!-- strip off the last ' or ' -->
                    <xsl:value-of select="substring($tmp, 0, string-length($tmp)-3)"/>
                </xsl:if>
            </xsl:variable>
            
            <xsl:if test="string-length($testsuiteQuery) > 0">
                <xsl:call-template name="restGET">
                    <xsl:with-param name="path"  select="'../suiteexecutionrecord'"/>
                    <xsl:with-param name="query" select="concat('fields=feed/entry/content/suiteexecutionrecord[', $testsuiteQuery, ']/(identifier|testplan)')"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:variable>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $tsers">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list>
                    <xsl:attribute name="key">RQM-TP-TSER-SECTION-CONTAINER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($tsers)/atom:feed/atom:entry/atom:content/rqm:suiteexecutionrecord[rqm:testplan/@href=$testplan/dc:identifier]"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Suite Results -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.planTestSuiteResults']">
        <xsl:param name="testplan"/>
        <xsl:variable name="isCurrentQuery">
            <xsl:text>isCurrent='true'</xsl:text>
        </xsl:variable>
        
        <xsl:variable name="tsresults">
            <!-- collect all the TSResults for this test plan via an additional REST API query -->
            <xsl:variable name="testsuiteQuery">
                <xsl:variable name="tmp">
                    <!-- build list of test suites that we need TS Results for -->
                    <xsl:for-each select="$testplan/rqm:testsuite">
                        <xsl:text>testsuite/@href='</xsl:text><xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, @href))"/><xsl:text>' or </xsl:text>
                    </xsl:for-each>    
                </xsl:variable>
                
                <xsl:if test="$tmp">
                    <!-- strip off the last ' or ' -->
                    <xsl:value-of select="substring($tmp, 0, string-length($tmp)-3)"/>
                </xsl:if>
            </xsl:variable>
            
            <xsl:if test="string-length($testsuiteQuery) > 0">
                <xsl:call-template name="restGET">
                    <xsl:with-param name="path"  select="'../testsuitelog'"/>
                    <xsl:with-param name="query" select="concat('fields=feed/entry/content/testsuitelog[ (', $testsuiteQuery, ') and ', $isCurrentQuery,  ']/(identifier)')"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:variable>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $tsresults">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list>
                    <xsl:attribute name="key">RQM-TP-TS_RESULTS-SECTION-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($tsresults)/atom:feed/atom:entry/atom:content/rqm:testsuitelog"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
        
    </xsl:template>

    <!-- Test Case Results -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.planTestCaseResults']">
        <xsl:param name="testplan"/>
            <xsl:variable name="isCurrentQuery">
                <xsl:text>isCurrent='true'</xsl:text>
            </xsl:variable>
            <xsl:variable name="testplanQuery">
            	<!-- build reference to testplan -->
                <xsl:text>testplan/@href='</xsl:text>
                <xsl:value-of select="utils:encodeUri($testplan/dc:identifier)"/>
                <xsl:text>'</xsl:text>
            </xsl:variable>
        	
        <xsl:variable name="tcresults">
            <!-- collect all the TCResults for this test plan via an additional REST API query -->
            
            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../executionresult'"/>
                <xsl:with-param name="query" select="concat('fields=feed/entry/content/executionresult[(', $testplanQuery,') and ', $isCurrentQuery, ']/(identifier)')"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $tcresults">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <table-list>
                    <xsl:attribute name="key">RQM-TP-TC_RESULTS-SECTION-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($tcresults)/atom:feed/atom:entry/atom:content/rqm:executionresult"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
        
    </xsl:template>
    
    
    <!-- Test Scripts -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planTestScripts']">
        <xsl:param name="testplan"/>
        
        <xsl:variable name="scripts">
            <!-- collect all the scripts for this test plan via an additional REST API query -->
            <xsl:variable name="testcaseQuery">
            
                <xsl:variable name="tmp">
                    <!-- build list of test cases that we need scripts for -->
                    <xsl:for-each select="$testplan/rqm:testcase">
                        <xsl:text>@href='</xsl:text><xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, @href))"/><xsl:text>' or </xsl:text>
                    </xsl:for-each>    
                </xsl:variable>
                
                <xsl:if test="$tmp">
                    <!-- strip off the last ' or ' -->
                    <xsl:value-of select="substring($tmp, 0, string-length($tmp)-3)"/>
                </xsl:if>
            </xsl:variable>
            
            <xsl:if test="string-length($testcaseQuery) > 0">
                <xsl:call-template name="restGET">
                    <xsl:with-param name="path"  select="'../testscript'"/>
                    <xsl:with-param name="query" select="concat('fields=feed/entry/content/testscript/(identifier|testcase[', $testcaseQuery, '])')"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:variable>
       
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $scripts">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>                
                <table-list>
                    <xsl:attribute name="key">RQM-TP-TS-SECTION-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($scripts)/atom:feed/atom:entry/atom:content/rqm:testscript[*]"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- This adds the proper node for an artifact within a table, with an attribute href= -->
    <xsl:template mode="collapse" match="rqm:executionworkitem|rqm:suiteexecutionrecord|rqm:testscript|rqm:testsuitelog|rqm:executionresult">
        <xsl:copy>
            <xsl:attribute name="href"><xsl:value-of select="dc:identifier"/></xsl:attribute>
        </xsl:copy>
    </xsl:template>
    
    <!-- Custom Grid Sections -->
    <xsl:template match="rqm:section[@type='GRID']">
        <xsl:param name="testplan"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testplan/*[@extensionDisplayName=$section-name]"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:copy-of select="proxy:gridJSONtoXHTML($item, $section-content)"/>    
        </xsl:copy>
    </xsl:template>
    
    <!-- Custom Sections -->
    <!-- This template is only invoked if no other template matched a rqm:section.  Any section that is not 
         explicitly handled above will simply be copied into the result document. -->
    <xsl:template match="rqm:section">
        <xsl:param name="testplan"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testplan/*[@extensionDisplayName=$section-name]/*"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $section-content">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$section-content"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>

 	<!-- eSignatures -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.esign.audit']">
        <xsl:param name="testplan"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$testplan/rqm:esignatures"/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>