<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:atom="http://www.w3.org/2005/Atom"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestSuiteProxy"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    exclude-result-prefixes="xsl xalan atom rqm dc alm exslt HttpClientExtension proxy utils log">
    
    <!-- included stylesheets -->
    <xsl:import href="cache://stylesheets/prep/abstract-prep-handler.xsl"/>
    
    <!-- params -->
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="catTypeRestService"/>
    
    <!-- proxy definition -->
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestSuiteProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $catTypeRestService)"/>
    
    <!-- templates -->
    <xsl:template match="rqm:testsuite">
        <xsl:copy>
            <xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:description"/>
            <xsl:apply-templates select="alm:owner"/>
            <xsl:apply-templates select="dc:creator"/>
            <xsl:apply-templates select="alm:updated"/>
            <xsl:apply-templates select="rqm:creationDate"/>
            <xsl:apply-templates select="rqm:priority"/>
            <xsl:apply-templates select="alm:state"/>
            <xsl:apply-templates select="rqm:copiedFrom"/>
            <xsl:call-template name="_insert-copiedTo"/>
            <xsl:for-each select="rqm:testplan">
            	<xsl:apply-templates select="utils:rest(concat(@href, '?fields=feed/entry/content/testplan/(', 'identifier|title', ')'))"/>
            </xsl:for-each>
            
            <xsl:if test="log:isDebugEnabled()">
                <xsl:value-of select="log:debug_xml(proxy:getSections($item))"/></xsl:if>
            
            <xsl:apply-templates select="proxy:getSections($item)/rqm:section">
                <xsl:with-param name="testsuite" select="current()"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="rqm:snapshot"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- copiedFrom -->
    <xsl:template match="rqm:copiedFrom">
        <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=/feed/entry/content/testsuite/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:testsuite">
        <prep:copiedFrom>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <!-- copiedTo -->
    <xsl:template name="_insert-copiedTo">
        <xsl:variable name="copiedToFeed">
            <xsl:variable name="tsFilter">copiedFrom/@href="<xsl:value-of select="utils:encodeUri(dc:identifier)"/>"</xsl:variable>

            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../testsuite'"/>
                <xsl:with-param name="query" select="concat('fields=/feed/entry/content/testsuite[', $tsFilter, ']/(identifier|title)')"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:apply-templates mode="copiedTo" select="exslt:node-set($copiedToFeed)//rqm:testsuite"/>
    </xsl:template>
    
    <xsl:template mode="copiedTo" match="rqm:testsuite">
        <prep:copiedTo>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedTo>
    </xsl:template>
    
    <!-- Summary -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.summary']">
        <xsl:param name="testsuite"/>
        <xsl:variable name="categories" select="proxy:getCategories($item)/rqm:category"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $categories or $testsuite/rqm:customAttributes">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$categories"/>
                <xsl:apply-templates select="$testsuite/rqm:customAttributes"/>
                <xsl:apply-templates select="$testsuite/rqm:estimate"/>
            	<xsl:apply-templates select="$testsuite/rqm:weight"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Formal Review -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.process.editor.section.review']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:approvals">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$testsuite/rqm:approvals"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Execution Variables -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.suiteExecVars']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:variables">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$testsuite/rqm:variables"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:section[@id='com.ibm.rqm.risk.editor.section.suiteAssessment']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:risk">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$testsuite/rqm:risk"/>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Cases -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.testcases']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:suiteelements">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list>
                    <xsl:attribute name="key">RQM-TS-TC-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates select="$testsuite/rqm:suiteelements/rqm:suiteelement"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Test Suite Execution Records -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.execution.editor.section.sers']">
        <xsl:param name="testsuite"/>
        
        <xsl:variable name="tserFeed">
            <xsl:variable name="query">
                <xsl:text>testsuite/@href='</xsl:text>
                <xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, $testsuite/dc:identifier))"/>
                <xsl:text>'</xsl:text>
            </xsl:variable>
            
            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../suiteexecutionrecord'"/>
                <xsl:with-param name="query" select="concat('fields=feed/entry/content/suiteexecutionrecord[', $query, ']/dc:identifier')"/>
            </xsl:call-template>
        </xsl:variable>
            
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $tserFeed">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <table-list>
                    <xsl:attribute name="key">RQM-TSER-SECTION-ID-TABLEVIEWER</xsl:attribute>
                    <xsl:apply-templates mode="collapse" select="exslt:node-set($tserFeed)/atom:feed/atom:entry/atom:content/rqm:suiteexecutionrecord"/>
                </table-list>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <xsl:template mode="collapse" match="rqm:suiteexecutionrecord">
        <xsl:copy>
            <xsl:attribute name="href"><xsl:value-of select="dc:identifier"/></xsl:attribute>
        </xsl:copy>
    </xsl:template>
    
    <!-- Attachments -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.attachments']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:attachment">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                
                <prep:attachments>
                    <xsl:for-each select="$testsuite/rqm:attachment">
                        <xsl:copy-of select="proxy:attachment($item, @href)"/>
                    </xsl:for-each>    
                </prep:attachments>
            </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Normative and Informative Documents -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.planNormativeInformative']">
        <xsl:param name="testsuite"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $testsuite/rqm:com.ibm.rqm.planning.editor.section.planNormativeInformative">
            <xsl:copy>
                <xsl:apply-templates select="@*|node()"/>
                <xsl:copy-of select="proxy:normativeAndInformativeJsonToXML($item, $testsuite/rqm:com.ibm.rqm.planning.editor.section.planNormativeInformative)"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
    
    <!-- Custom Grid Sections -->
    <xsl:template match="rqm:section[@type='GRID']">
        <xsl:param name="testsuite"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testsuite/*[@extensionDisplayName=$section-name]"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:copy-of select="proxy:gridJSONtoXHTML($item, $section-content)"/>    
        </xsl:copy>
    </xsl:template>
    
    <!-- Custom Sections -->
    <!-- This template is only invoked if no other template matched a rqm:section.  Any section that is not 
         explicitly handled above will simply be copied into the result document. -->
    <xsl:template match="rqm:section">
        <xsl:param name="testsuite"/>
        <xsl:variable name="section-name" select="@name"/>
        <xsl:variable name="section-content" select="$testsuite/*[@extensionDisplayName=$section-name]/*"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $section-content">
            <xsl:copy>
                <xsl:apply-templates select="@*"/>
                <xsl:apply-templates select="$section-content"/>
            </xsl:copy>    
        </xsl:if>
    </xsl:template>
    
     <!-- eSignatures -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.esign.audit']">
        <xsl:param name="testsuite"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$testsuite/rqm:esignatures"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>