<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:result="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xsl xalan rqm dc alm result time utils">
    
    <!-- included stylesheets -->
    <xsl:import  href="cache://stylesheets/render/abstract-document.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/utils.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/icons/result-handler.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/lab-resource.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/summary-section.xsl"/>
    <xsl:include href="cache://stylesheets/render/execution-result/table.xsl"/>
    <xsl:include href="cache://stylesheets/render/work-item/table.xsl"/>
    <xsl:include href="cache://stylesheets/render/attachment/table.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/execution-variables.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/formal-review.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/eSignatures.xsl"/>
    
    <xsl:variable name="header-icon" select="'basecache://com.ibm.asq.common.web/resources/ui/internal/images/icons/view16/qm_execresult48.gif'"/>
    
    <!-- templates -->
    <xsl:template match="/rqm:executionresult">
        <xsl:call-template name="buildTitleSection">
            <xsl:with-param name="name" select="dc:title"/>
			<xsl:with-param name="id" select="rqm:webId"/>
			<xsl:with-param name="href" select="dc:identifier"/>
			<xsl:with-param name="header-icon" select="$header-icon"/>
        </xsl:call-template>
        
        <xsl:call-template name="_buildDetailsSection"/>
        
        <xsl:apply-templates select="rqm:section"/>
    </xsl:template>
    
    <!-- Execution Result Summary -->
    <xsl:template name="_buildDetailsSection">
        <fo:table>
            <fo:table-column column-width="18%"/>
            <fo:table-column column-width="2%" /> <!-- Icon -->
            <fo:table-column column-width="31%"/>
            <fo:table-column column-width="18%"/>
            <fo:table-column column-width="31%"/>
            <fo:table-body>
                <!-- Approval State -->
                <fo:table-row>
                    <fo:table-cell>
                    	<xsl:variable name="stateId" select="result:approvalstate"/>                  
                        <xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
                       	<fo:block>
	                       	<xsl:value-of select="$strings/State"/>: 
	                       	<xsl:text> </xsl:text>
                        	<xsl:choose>
                        		<xsl:when test="$stateName">
                        		    <fo:inline xsl:use-attribute-sets="value"><xsl:value-of select="$stateName"/></fo:inline>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:variable name="workflowType" select="'TESTCASERESULT'"/>    
                        			<fo:inline xsl:use-attribute-sets="value"><xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/></fo:inline>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row height="8pt">
                    <fo:table-cell><fo:block linefeed-treatment="preserve"/></fo:table-cell>
                </fo:table-row>
                <!-- Actual Result -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Actual_Result"/>:</fo:block></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:call-template name="printResultStateIcon">
                                <xsl:with-param name="size">45%</xsl:with-param>
                                <xsl:with-param name="priority_id"><xsl:value-of select="alm:state"/></xsl:with-param>
                            </xsl:call-template>
                        </fo:block>
                    </fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:apply-templates select="alm:state"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Host Name -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Host_Name"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="result:machine"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Creator -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Creator"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:apply-templates select="prep:creator/@username"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Owner -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Owner"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:apply-templates select="prep:owner"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Tested by -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/TestedBy"/></fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                    	<fo:block xsl:use-attribute-sets="value">
	                     	<xsl:for-each select="result:testedby">
	                          <xsl:apply-templates select="prep:tester/@username"/>
	                          <xsl:if test="not(position()=last())">
	                              <xsl:text>, </xsl:text>
	                          </xsl:if>
	                         </xsl:for-each>
	                    </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row height="8pt">
                    <fo:table-cell><fo:block linefeed-treatment="preserve"/></fo:table-cell>
                </fo:table-row>
                
                <!-- Test Plan -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Plan"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:if test="rqm:testplan">
                                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:testplan/dc:identifier}">
                                    <xsl:value-of select="concat(rqm:testplan/rqm:webId, ': ', rqm:testplan/dc:title)"/>    
                                </fo:basic-link>
                            </xsl:if>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row> 
                <!-- Iteration -->        <!-- Start Time -->
				<fo:table-row>
					<fo:table-cell>
						<fo:block xsl:use-attribute-sets="label"><xsl:value-of select="$strings/Iteration"/>:</fo:block>
					</fo:table-cell>
					 <fo:table-cell><fo:block /></fo:table-cell>
					<fo:table-cell>
						<fo:block xsl:use-attribute-sets="value">
							<xsl:choose>
								<xsl:when test="rqm:testphase">
									<xsl:value-of select="rqm:testphase/dc:title"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$strings/Unassigned"/>
								</xsl:otherwise>
							</xsl:choose>
						</fo:block>
					</fo:table-cell>
					
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Start_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="time:formatMediumDateMediumTime(result:starttime, $locale)"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Test Case -->             <!-- End Time -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Case"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:if test="rqm:testcase">
                                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:testcase/dc:identifier}">
                                    <xsl:value-of select="concat(rqm:testcase/rqm:webId, ': ', rqm:testcase/dc:title)"/>    
                                </fo:basic-link>
                            </xsl:if>
                        </fo:block>
                    </fo:table-cell>
                    
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/End_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="time:formatMediumDateMediumTime(result:endtime, $locale)"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Test Script -->           <!-- Total Run Time -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Script"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:if test="rqm:executionscript">
                                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:executionscript/dc:identifier}">
                                    <xsl:value-of select="concat(rqm:executionscript/rqm:webId, ': ', rqm:executionscript/dc:title)"/>    
                                </fo:basic-link>
                            </xsl:if>
                        </fo:block>
                    </fo:table-cell>
                    
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Total_Run_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block><xsl:value-of select="time:formatTimeEstimate(result:starttime, result:endtime, $locale)"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Test Data -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Data"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:choose>
                                <xsl:when test="rqm:datapool">
                                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:datapool/dc:identifier}">
                                        <xsl:value-of select="rqm:datapool/dc:title"/>    
                                    </fo:basic-link>
                                </xsl:when>
                                <xsl:otherwise>
                                    <fo:inline xsl:use-attribute-sets="value">
                                        <xsl:value-of select="$strings/Unassigned"/>
                                    </fo:inline>
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Test Data Row Numbers-->	        
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Data_Rows"/></fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                    	<fo:block xsl:use-attribute-sets="value">
                    	   <xsl:choose>
                              <xsl:when test="rqm:testdatarows">
		                     	<xsl:for-each select="rqm:testdatarows/rqm:rownumber">
		                          <xsl:value-of select="."></xsl:value-of>
		                          	<xsl:if test="not(position()=last())">
                                    	<xsl:text>, </xsl:text>
                                	</xsl:if>
		                         </xsl:for-each>
	                          </xsl:when>
                              <xsl:otherwise>                                    
                                        <xsl:value-of select="$strings/Unassigned"/>
                              </xsl:otherwise>
                           </xsl:choose>
	                    </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                                
                <!-- Build -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Build"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:choose>
                                <xsl:when test="rqm:buildrecord">
                                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:buildrecord/dc:identifier}">
                                       <xsl:choose>
                                			<xsl:when test="rqm:builddefinition">
                                				<xsl:value-of select="concat(rqm:buildrecord/rqm:webId, ': ', rqm:builddefinition/dc:title, ' : ', rqm:buildrecord/dc:title)"/>    
                                        	</xsl:when>
                                			<xsl:otherwise>
                                        		<xsl:value-of select="concat(rqm:buildrecord/rqm:webId, ': ', rqm:buildrecord/dc:title)"/>    
                                			</xsl:otherwise>
			                            </xsl:choose>
                                    </fo:basic-link>
                                </xsl:when>
                                <xsl:otherwise>
                                    <fo:inline xsl:use-attribute-sets="value">
                                        <xsl:value-of select="$strings/Unassigned"/>
                                    </fo:inline>
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Weight -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Weight"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block /></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="result:weight"/></fo:block></fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
        
        <!--<fo:block xsl:use-attribute-sets="artifact-description"><xsl:value-of select="$strings/Description"/>: <fo:inline xsl:use-attribute-sets="value"><xsl:apply-templates select="rqm:executionresult/dc:description"/></fo:inline></fo:block>-->
    </xsl:template>
    
    <xsl:template match="result:result|alm:state">
        <xsl:variable name="stateId" select="."/>
        <xsl:variable name="stateValue" select="@value"/>
        
        <fo:inline>
            <xsl:choose>
        		<xsl:when test="$stateValue">
        		    <!-- a resolved state name -->
        	 		<xsl:value-of select="$stateValue"/>
        		</xsl:when>
        		<xsl:otherwise>
        			<!-- still an unresolved state id -->
        			<xsl:value-of select="$strings/*[name()=$stateId]"/>
        		</xsl:otherwise>
        	</xsl:choose>
        </fo:inline>
    </xsl:template>
    
    <xsl:template match="rqm:section">
        <!-- TODO: for debugging only -->
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Weight Distribution']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Weight_Distribution"/></fo:block>
        
        <!-- section content -->
        <fo:block><xsl:value-of select="$strings/Pass"/>: <xsl:value-of select="result:pointspassed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Fail"/>: <xsl:value-of select="result:pointsfailed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Inconclusive"/>: <xsl:value-of select="result:pointsinconclusive"/></fo:block>
        <fo:block><xsl:value-of select="$strings/PermFailed"/>: <xsl:value-of select="rqm:pointspermfailed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Deferred"/>: <xsl:value-of select="rqm:pointsdeferred"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Blocked"/>: <xsl:value-of select="result:pointsblocked"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Attempted"/>: <xsl:value-of select="result:pointsattempted"/></fo:block>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Test Environment']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Test_Environment"/></fo:block>
        
        <!-- section content -->
        <xsl:if test="rqm:configuration">
            <fo:block>
                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:configuration/dc:identifier}">
                    <xsl:value-of select="rqm:configuration/dc:title"/>
                </fo:basic-link>
            </fo:block>
            
            <xsl:call-template name="buildResourceFromJSON">
                <xsl:with-param name="configuration-json" select="rqm:configuration/rqm:configuration"/>
            </xsl:call-template>    
        </xsl:if>
        
        <fo:block>
            <fo:inline xsl:use-attribute-sets="label"><xsl:value-of select="$strings/Channel"/>:</fo:inline>
            <xsl:text> </xsl:text>
            <fo:inline xsl:use-attribute-sets="value"><xsl:value-of select="rqm:channel/dc:title"/></fo:inline>
        </fo:block>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Categories']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Categories"/></fo:block>
        
        <!-- section content -->
        <xsl:call-template name="buildSummarySection"/> <!-- handled by summary-section.xsl -->
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Previous Results']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Previous_Results"/></fo:block>
        
        <!-- section content -->
        <xsl:variable name="key" select="table-list/@key"/>
        <xsl:variable name="columns" select="$renderingDetails/prep:table[@key=$key]/prep:column"/>
        
        <xsl:call-template name="buildExecutionResultTable">
			<xsl:with-param name="columns" select="$columns"/>
            <xsl:with-param name="results" select="table-list/rqm:executionresult"/>
        </xsl:call-template>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Defects']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Defects"/></fo:block>
        
        <!-- section content -->
        <xsl:call-template name="buildCalmWorkItemTable">
            <xsl:with-param name="strings" select="$strings"/>
            <xsl:with-param name="item-list" select="rqm:defect"/>
            
            <xsl:with-param name="render-relationship" select="true()"/>
            <xsl:with-param name="render-summary" select="true()"/>
            <xsl:with-param name="link" select="true()"/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Notes -->
    <xsl:template match="rqm:section[@name='Notes']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Notes"/></fo:block>
        
        <!-- section content -->
        <fo:block><xsl:apply-templates/></fo:block>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Result Details']">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Result_Details"/></fo:block>
        
        <!-- section content -->
        <!-- Note: The Result Details section contains result:stepResults when a test script is executed and result:details when a test case is executed. -->
        <xsl:choose>
    		<xsl:when test="result:stepResults/result:stepResult">
		        <xsl:call-template name="_buildExeResultStepDetails">
		            <xsl:with-param name="result-list" select="result:stepResults/result:stepResult"/>
		        </xsl:call-template>
		    </xsl:when>
    		<xsl:otherwise>
				<fo:block><xsl:apply-templates/></fo:block>
 			</xsl:otherwise>
		</xsl:choose>
    </xsl:template>
    
    <xsl:template name="_buildExeResultStepDetails">
        <xsl:param name="result-list"/>
        
        <!-- print each result -->
        <xsl:for-each select="$result-list">
            <xsl:choose>
                <!-- keyword steps -->
                <xsl:when test="result:stepType='com.ibm.rqm.execution.common.elementtype.keyword'">
                    <fo:block xsl:use-attribute-sets="keyword-block">
                        <!-- name -->
                        <fo:block xsl:use-attribute-sets="step-name">
                            <xsl:value-of select="result:description"/>
                        </fo:block>
                        
                        <!-- type -->
                        <fo:block xsl:use-attribute-sets="step-type"><xsl:apply-templates select="result:stepType"/></fo:block>
                        
                        <!-- sub-steps -->
                        <xsl:call-template name="_buildExeResultStepDetails">
                            <xsl:with-param name="result-list" select="result:stepResults/result:stepResult"/>
                        </xsl:call-template>
                    </fo:block>
                </xsl:when>
                <!-- execution steps -->
                <xsl:otherwise>
                    <fo:block xsl:use-attribute-sets="step-block">                
                        <!-- name -->
                        <!-- testdata row number -->
                        <fo:block xsl:use-attribute-sets="step-name" text-align-last="justify">	                        
                            <xsl:if test="result:testdatarownumber">
                        		<xsl:value-of select="$strings/Test_Data_Row"/><xsl:text> : </xsl:text><xsl:value-of select="result:testdatarownumber"/>
                        		<fo:leader leader-pattern="space"/>
                        	</xsl:if>
                            <xsl:value-of select="$strings/Step"/><xsl:text> </xsl:text><xsl:value-of select="@stepIndex"/>
                            <fo:leader leader-pattern="space"/>
                            <xsl:call-template name="printResultStateIcon">
                                <xsl:with-param name="priority_id"><xsl:value-of select="result:result"/></xsl:with-param>
                                <xsl:with-param name="size">70%</xsl:with-param>
                            </xsl:call-template>
                            <xsl:text> </xsl:text>
                            <xsl:apply-templates select="result:result"/>
                        </fo:block>
                        
                        <!-- type -->
                        <fo:block xsl:use-attribute-sets="step-type"><xsl:apply-templates select="result:stepType"/></fo:block>
                        
                        <!-- start time -->                                <!-- end time -->
                        <fo:block>
                            <xsl:value-of select="$strings/Start_Time"/>: <xsl:value-of select="time:formatMediumDateMediumTime(result:startTime, $locale)"/>
                            <fo:leader leader-pattern="space"/><fo:leader leader-pattern="space"/>
                            <xsl:value-of select="$strings/End_Time"/>: <xsl:value-of select="time:formatMediumDateMediumTime(result:endTime, $locale)"/>
                        </fo:block>
                                                                        
                        <!-- description -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/Description"/></fo:block>
                        <fo:block><xsl:apply-templates select="result:description"/></fo:block>
                        
                        <!-- expected result -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/Expected_Results"/></fo:block>
                        <fo:block><xsl:apply-templates select="result:expectedResult"/></fo:block>
                        
                        <!-- actual result -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/Actual_Results"/></fo:block>
                        <fo:block><xsl:apply-templates select="result:actualResult"/></fo:block>
                        
                        <!-- comments -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/Comments"/></fo:block>
                        <fo:block><xsl:apply-templates select="result:comment"/></fo:block>
                        
                        <!-- Tested By column -->
                        <fo:block xsl:use-attribute-sets="step-header"><xsl:value-of select="$strings/TestedBy"/></fo:block>
                        <fo:block><xsl:apply-templates select="prep:tester/@username"/></fo:block>
                        
                        <!-- work items -->
                        <fo:block space-before="8pt">
                            <xsl:value-of select="$strings/Defects"/>: 
                            <xsl:for-each select="result:defect">
                                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{@href}">
                                    <xsl:apply-templates select="@summary"/>
                                </fo:basic-link>
                                <xsl:if test="not(position()=last())">
                                    <xsl:text>, </xsl:text>
                                </xsl:if>
                            </xsl:for-each>
                        </fo:block>
                        
                        <!-- attachments -->
						<fo:block>
							<xsl:value-of select="$strings/Attachments" />:
							<xsl:call-template name="buildAttachmentList">
								<xsl:with-param name="attachments" select="result:stepAttachment" />
							</xsl:call-template>
						</fo:block>
					</fo:block>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>
    
    <xsl:template match="result:stepType">
        <xsl:choose>
            <!-- execution step -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.elementtype.execution'">
                <xsl:value-of select="$strings/Execution_Step"/></xsl:when>
            <!-- keyword step -->
            <xsl:when test="current()='com.ibm.rqm.execution.common.elementtype.keyword'">
                <xsl:value-of select="$strings/Keyword_Step"/></xsl:when>
            <!-- default -->
            <xsl:otherwise>
                <xsl:value-of select="current()"/></xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Audit']">
        <xsl:variable name="testcasestateatexecutionstateid" select="result:testcasestateatexecution"/>
        <xsl:variable name="testplanstateatexecutionstateid" select="result:testplanstateatexecution"/>
        <xsl:variable name="testscriptstateatexecutionstateid" select="result:testscriptstateatexecution"/>
        <xsl:if test="$testcasestateatexecutionstateid">
	        <!-- section header -->
	        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Audit"/></fo:block>
	        
	        <!-- section content -->
	        <fo:block><xsl:value-of select="$strings/DateWhenRun"/><xsl:value-of select="time:formatMediumDateMediumTime(result:starttime, $locale)"/></fo:block>
            <xsl:if test="$testplanstateatexecutionstateid">
		        <xsl:variable name="testplanstateatexecutionstatename" select="$strings/*[name()=$testplanstateatexecutionstateid]"/>    
				<fo:block>
		        	<xsl:value-of select="$strings/TestPlanStateAtExecution"/>: 
		            <xsl:text> </xsl:text>
	                <xsl:choose>
		                <xsl:when test="$testplanstateatexecutionstatename">
		                	<xsl:value-of select="$testplanstateatexecutionstatename"/>
		                </xsl:when>
		                <xsl:otherwise>
		                	<xsl:variable name="workflowType" select="'TESTPLAN'"/>    
		                    <xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testplanstateatexecutionstateid)"/>
		                </xsl:otherwise>
		            </xsl:choose>
	            </fo:block>
	        </xsl:if>
	        <xsl:variable name="testcasestateatexecutionstatename" select="$strings/*[name()=$testcasestateatexecutionstateid]"/>    
			<fo:block>
	        	<xsl:value-of select="$strings/TestCaseStateAtExecution"/>: 
	            <xsl:text> </xsl:text>
                <xsl:choose>
	                <xsl:when test="$testcasestateatexecutionstatename">
	                	<xsl:value-of select="$testcasestateatexecutionstatename"/>
	                </xsl:when>
	                <xsl:otherwise>
	                	<xsl:variable name="workflowType" select="'TESTCASE'"/>    
	                    <xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testcasestateatexecutionstateid)"/>
	                </xsl:otherwise>
	            </xsl:choose>
            </fo:block>
            <xsl:if test="$testscriptstateatexecutionstateid">
		        <xsl:variable name="testscriptstateatexecutionstatename" select="$strings/*[name()=$testscriptstateatexecutionstateid]"/>    
				<fo:block>
		        	<xsl:value-of select="$strings/TestScriptStateAtExecution"/>: 
		            <xsl:text> </xsl:text>
	                <xsl:choose>
		                <xsl:when test="$testscriptstateatexecutionstatename">
		                	<xsl:value-of select="$testscriptstateatexecutionstatename"/>
		                </xsl:when>
		                <xsl:otherwise>
		                	<xsl:variable name="workflowType" select="'TESTSCRIPT'"/>    
		                    <xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testscriptstateatexecutionstateid)"/>
		                </xsl:otherwise>
		            </xsl:choose>
	            </fo:block>
            </xsl:if>
        </xsl:if>
    </xsl:template>
   
</xsl:stylesheet>