<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2015. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:exsl="http://exslt.org/common"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:pref_table="http://com.ibm.rqm.export.pdf/preferences/table"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    exclude-result-prefixes="xsl exsl rqm dc alm prep utils time"
    extension-element-prefixes="exsl">
    
    <!-- included templates -->
    <xsl:import  href="cache://stylesheets/render/abstract-table-summary.xsl"/>
    
    <!-- for debugging only -->
    <!--<xsl:include href="cache://stylesheets/render/_config/attribute-sets.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/utils.xsl"/>-->
    
    <!-- templates -->
    <xsl:template name="buildSuiteElementTable">
        <xsl:param name="columns"/>
        <xsl:param name="suiteElements"/>
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(pref_table:width, '%')}"/>
            </xsl:for-each>
            
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="translate(pref_table:name, ' ', '_')"/>
                        <fo:block>
                            <xsl:choose>
                                <xsl:when test="starts-with($header, 'isCategoryColumn')">
                                    <xsl:value-of select="substring-after($header, 'isCategoryColumn')"/>
                                </xsl:when>
                                <xsl:when test="$header='ID'">
                                    <fo:block xsl:use-attribute-sets="id-header"><xsl:value-of select="$strings/id"/></fo:block>
                                </xsl:when>
                                <xsl:when test="$header='Priority'">
                                    <fo:block xsl:use-attribute-sets="priority-header"><xsl:value-of select="$strings/Priority"/></fo:block>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$strings/*[name()=$header]"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>
            
            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$suiteElements">
                        <xsl:for-each select="$suiteElements">
                            <xsl:variable name="suiteElement" select="current()"/>
                            <fo:table-row>
                                <xsl:for-each select="$columns">
                                    <xsl:apply-templates select="current()">
                                        <xsl:with-param name="artifact" select="$suiteElement"/>
                                    </xsl:apply-templates>
                                </xsl:for-each>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts empty columns so that the table header can still appear in the document -->
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <!-- Execution Order -->
    <xsl:template match="pref_table:column/pref_table:name[.='Execution Order']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:value-of select="$artifact/@elementindex+1"/></fo:block>
    </xsl:template>
    
    <!-- ID -->
    <xsl:template match="pref_table:column/pref_table:name[.='ID']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <fo:block xsl:use-attribute-sets="id-cell">
            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testcase/dc:identifier}">
                <xsl:value-of select="$testcase/rqm:webId"/>
            </fo:basic-link>
        </fo:block>
    </xsl:template>
    
    <!-- Priority -->
    <xsl:template match="pref_table:column/pref_table:name[.='Priority']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <fo:block xsl:use-attribute-sets="priority-cell">
            <xsl:apply-templates select="$testcase/rqm:priority" />
        </fo:block>  
    </xsl:template>
    
    <!-- Name -->
    <xsl:template match="pref_table:column/pref_table:name[.='Test Case']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <fo:block>
            <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$testcase/dc:identifier}">
                <xsl:apply-templates select="$testcase/dc:title"/>
            </fo:basic-link>
        </fo:block>
    </xsl:template>
    
    <!-- Modified -->
    <xsl:template match="pref_table:column/pref_table:name[.='Modified']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <fo:block><xsl:value-of select="time:formatMediumDate($testcase/alm:updated, $locale)"/></fo:block>
    </xsl:template>
    
    <!-- State -->
    <xsl:template match="pref_table:column/pref_table:name[.='State']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
         
        <xsl:variable name="stateId" select="$testcase/alm:state"/>                  
		<xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
        <fo:block>
            <xsl:choose>
            	<xsl:when test="$stateName">
                	<xsl:value-of select="$stateName"/>
				</xsl:when>
                <xsl:otherwise>
                	<xsl:variable name="workflowType" select="'TESTCASE'"/>    
					<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
                </xsl:otherwise>
            </xsl:choose>
        </fo:block>            
    </xsl:template>
    
    <!-- Owner -->
    <xsl:template match="pref_table:column/pref_table:name[.='Owner']">
        <xsl:param name="artifact"/>
        
        <fo:block><xsl:apply-templates select="$artifact/prep:owner"/></fo:block>
    </xsl:template>
    
    <!-- Estimate -->
    <xsl:template match="pref_table:column/pref_table:name[.='Estimate']">
        <xsl:param name="artifact"/>
        <xsl:variable name="testcase" select="$artifact/rqm:testcase"/>
        
        <xsl:choose>
            <xsl:when test="$testcase/rqm:estimate">
                <fo:block><xsl:value-of select="time:formatTimeEstimate($testcase/rqm:estimate, $locale)"/></fo:block>
            </xsl:when>
            <xsl:otherwise>
                <fo:block></fo:block>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Test Script -->
    <xsl:template match="pref_table:column/pref_table:name[.='Test Script']">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="script" select="$artifact/rqm:executionscript"/>
        
        <fo:block>
        	<xsl:if test="$script">
            	<fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$script/dc:identifier}">
                	<xsl:value-of select="$script/dc:title"/>
                </fo:basic-link>    
            </xsl:if>
        </fo:block>
    </xsl:template>
    
    <!-- Test Environment -->
    <xsl:template match="pref_table:column/pref_table:name[.='Test Environment']">
        <xsl:param name="artifact"/>
        <xsl:variable name="configuration" select="$artifact/rqm:configuration"/>
        
        <fo:block>
            <xsl:if test="$configuration">
                <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{$configuration/dc:identifier}">
                    <xsl:value-of select="$configuration/dc:title"/>
                </fo:basic-link>
            </xsl:if>
        </fo:block>
    </xsl:template>
    
</xsl:stylesheet>