<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
  * (c) Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    exclude-result-prefixes="xsl rqm dc prep log time"
    extension-element-prefixes="log time">
    
    <xsl:import href="cache://stylesheets/render/abstract-table-summary.xsl"/>
    
    <xsl:template name="buildTestPhaseTable">
        <xsl:param name="columns"/>
        <xsl:param name="testphases"/>
        
        
        <fo:table xsl:use-attribute-sets="artifact-table">
            
            <xsl:for-each select="$columns">
                <fo:table-column column-width="{concat(prep:width, '%')}"/>
            </xsl:for-each>
            
            
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <xsl:for-each select="$columns[not(prep:name='actions')]">
                    <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                        <xsl:variable name="header" select="prep:name"/>
                        <fo:block>
                            <xsl:choose>
                                <xsl:when test="starts-with($header, 'isCategoryColumn')">
                                    <xsl:value-of select="substring-after($header, 'isCategoryColumn')"/></xsl:when>
                                <xsl:when test="starts-with($header, 'isCustomColumn')">
                                    <xsl:value-of select="substring-after($header, 'isCustomColumn')"/></xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$strings/*[name()=$header]"/></xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </xsl:for-each>
            </fo:table-header>
            
            
            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$testphases">
                        <xsl:apply-templates select="$testphases">
                            <xsl:with-param name="columns" select="$columns"/>
                        </xsl:apply-templates>
                    </xsl:when>
                    <xsl:otherwise>
                        <!-- inserts empty columns so that the table header can still appear in the document -->
                        <fo:table-row>
                            <xsl:for-each select="$columns">
                                <fo:table-cell><fo:block/></fo:table-cell>
                            </xsl:for-each>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    <!-- Table Rows (Test Phases) -->
    <xsl:template match="rqm:testphase">
        <xsl:param name="columns"/>
        
        <fo:table-row>
            <xsl:apply-templates mode="phase-value" select="$columns">
                <xsl:with-param name="artifact" select="current()"/>
            </xsl:apply-templates>
        </fo:table-row>
        <xsl:apply-templates>
            <xsl:with-param name="columns" select="$columns"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template mode="phase-value" match="prep:column">
        <xsl:param name="artifact"/>
        
        <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell">
        	<fo:block-container overflow="hidden">
	            <!--  match type-specific value templates locally defined in this file  -->
	            <xsl:apply-templates mode="phase-value" select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
	            <!--  and now match generic value templates from abstract-table-summary.xsl  -->
	            <xsl:apply-templates select="prep:name">
	                <xsl:with-param name="artifact" select="$artifact"/>
	            </xsl:apply-templates>
            </fo:block-container>
        </fo:table-cell>
    </xsl:template>
    
    <!-- Table Columns -->
    <!-- Name -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='name']">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="title" select="concat($artifact/dc:title, ' [', 
            time:formatShortDate($artifact/rqm:starttime, $locale), ' - ',
            time:formatShortDate($artifact/rqm:endtime, $locale), ']')"/>
        
        <fo:block text-indent="{$artifact/prep:level * 10}">
            <xsl:value-of select="$title"/>
        </fo:block>
    </xsl:template>
    
    <!-- Description -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='description']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:apply-templates select="$artifact/dc:description"/>
        </fo:block>
    </xsl:template>
    
    <!-- Points -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='points']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:apply-templates select="$artifact/rqm:expectedTotalPoints"/>
        </fo:block>
    </xsl:template>
    
    <!-- Planned Defects -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='planned_defects']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:apply-templates select="$artifact/rqm:expectedDefects"/>
        </fo:block>
    </xsl:template>
    
    <!-- Start Date -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='startdate']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:value-of select="time:formatMediumDate($artifact/rqm:expectedStartDate, $locale)"/>
        </fo:block>
    </xsl:template>
    
    <!-- End Date -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='endDate']">
        <xsl:param name="artifact"/>
        
        <fo:block>
            <xsl:value-of select="time:formatMediumDate($artifact/rqm:expectedEndDate, $locale)"/>
        </fo:block>
    </xsl:template>
    
    <!-- Duration -->
    <xsl:template mode="phase-value" match="prep:column/prep:name[.='duration']">
        <xsl:param name="artifact"/>
        
        <fo:block>
        	<xsl:choose>
        		<xsl:when test="$artifact/rqm:expectedStartDate and $artifact/rqm:expectedEndDate">
        			<xsl:value-of select="time:personDaysBetween($artifact/rqm:expectedStartDate, $artifact/rqm:expectedEndDate, $locale)"/>
        		</xsl:when>
        		<xsl:otherwise>
        			<xsl:value-of select="time:personDays('0', $locale)"/>
        		</xsl:otherwise>
        	</xsl:choose>
        </fo:block>
    </xsl:template>
    
    <!-- Actions -->
    <xsl:template mode="phase-value" match="prep:column[prep:name='actions']">
        <!-- Not Supported -->
    </xsl:template>
    
</xsl:stylesheet>