/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamMultiSelect");

dojo.require("dojo.i18n");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.SimpleToolbar");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");
dojo.require("com.ibm.team.reports.web.ui.internal.ParamValuesSelectionDialog");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	var ParamValuesSelectionDialog = com.ibm.team.reports.web.ui.internal.ParamValuesSelectionDialog;
	var MAX_INITIAL_DEFAULT_VALUES = 50;
	var ADD_VALUES_BUTTON_ID = "addValuesButtonId";
	var SELECT_ALL = "'*'";
	
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamMultiSelect", [com.ibm.team.reports.web.ui.internal.ReportParam], {
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamMultiSelect.html"),
		
		valuesMouseOver: null,
		valuesMouseOut: null,
		
		constructor: function() {
			this._numSelectedValues = 0;
			this._numValues = 0;
			this._numTotalValues = 0;
			this._addedReportParameterDTOs = [];  //the DTOs corresponding to the visible parameter values
			this._liNodeLevels = [];
			this._dupCheckBoxes = [];
			this._tooltip = null;
			// A map of checkbox ids to list items
			this._idToListItem = [];
			this._arrayOfItems = [];
			this._arrayOfHierarchyItems = [];
			this._defaultValues = [];
			this._additionalValues = [];
		
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			
			this._localizedLabels = [];
			this._localizedLabels[this.statics.MY_TEAM_AREAS] = this.messages.myTeamAreas;
			this._localizedLabels[this.statics.CURRENT_ITERATION] = this.messages.currentIteration;
			this._localizedLabels[this.statics.CURRENT_CONTRIBUTOR] = this.messages.currentContributor;
		},
		
		postCreate: function() {
			this._toolbar = new jazz.ui.SimpleToolbar({});
			this._toolbarDiv.appendChild(this._toolbar.domNode);

			this._toolbar.addButton(this, this._selectAll, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/selectall.gif"), label: this.messages.selectAll});
			this._toolbar.addButton(this, this._deselectAll, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/deselectall.gif"), label: this.messages.deSelectAll});
			this._toolbarDiv.style.display = "none";
		},
		
		uninitialize: function() {
			if (this._tooltip != null) {
				this._tooltip.destroy();
				this._tooltip = null;
			}
			
			var listItem = this.checkBoxList.firstChild;
			while (listItem) {
				var checkBox = listItem.props.checkBoxChild;
				if (checkBox) {
					dojo.destroy(checkBox);
				}
				listItem = listItem.nextSibling;
			}
			this._idToListItem = [];
			this._rootListItems = [];
			this._removeChildren(this.checkBoxList);
			dojo.empty(this.specialParamSpan);
			this._specialCheckBox = null;

			this._numValues = 0;
			this._numSelectedValues = 0;
			this._defaultValues = [];
			this._additionalValues = [];
			this._arrayOfItems = [];
			this._arrayOfHierarchyItems = [];
			this._reportParameterDTOs = [];
//			this.clear(); //necessary?
		},
		
		_selectAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (!checkBox.checked && !checkBox.disabled && (!dojo.hasClass(this._idToListItem[checkBox.id], "display-none") || checkBox.props.isSpecial)) {
					checkBox.checked = true;
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true, propagate: false});
				}
			}));
			
			if (this.childReportParam) {
				var reportParameterDTOsToAdd = [];
				var values;
				if (this.isRoot) {
					values = this._paramDTO.values;
					for (var j = 0; j < values.length; ++j) {
						if (this._paramDTO.children[j]) {
							reportParameterDTOsToAdd.push(this._paramDTO.children[j]);
						}
					}
				} else {
					values = this._addedReportParameterDTOs;
					for (var j = 0; j < values.length; ++j) {
						if (values[j].children) {
							for(var k = 0; k < values[j].children.length; ++k){
								reportParameterDTOsToAdd.push(values[j].children[k]);
							}
						}
					}
				}
				this.childReportParam.cascadeClear();
				this.childReportParam.addValues(reportParameterDTOsToAdd);
			}
			
			this._updateHeaderSelectedStats();
		},
		
		_deselectAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (checkBox.checked && (!dojo.hasClass(this._idToListItem[checkBox.id], "display-none") || checkBox.props.isSpecial)) {
					checkBox.checked = false;
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true, propagate: false});
				}
			}));
			
			if (this.childReportParam) {
				this._setAllOrNoneSelections();
			}
			this._updateHeaderSelectedStats();
		},
		
		_handleAddValuesAction: function() {
			var self= this;
			var params= {
					dialogTitle: this.messages.addValuesDialogTitle,
					allowMultiple: true,
					useOkCancel: false, /* Not Add/Close */
					removeUsersOnAdd: true,
					filterOutUsers: [], 
					onOK: function(newValues) {
						if(newValues && newValues.length > 0)
							self._addMoreValuesToList(newValues);
					},
					values: this._additionalValues
				};
			new ParamValuesSelectionDialog(params);
		},
		
		_addMoreValuesToList: function(newValues){
			for (var i = 0, j = this._defaultValues.length; i < newValues.length; i++) {
				var value = newValues[i].itemId;
				var label = newValues[i].label;
				this._appendCheckBox(this._paramDTO, value, label);
				this._defaultValues[j++] = [this._paramDTO, value, label];
				
				var listLength = this._arrayOfItems.length;
				if(listLength - this.checkBoxList.childNodes.length == 1) {
					this.checkBoxList.appendChild(this._arrayOfItems[listLength - 1].item);
					this._updateHeaderSelectedStats();
				}
				
				//remove this value from this._additionalValues
				for(var v = 0; v < this._additionalValues.length; v++) {
					if(value == this._additionalValues[v].itemId)
						this._additionalValues.splice(v, 1);
				}
			}
			
			/*
			 * If the user added hierarchical items, we don't know where in the list they were added,
			 * or how many were added (ex. if one 3rd-level node was added by the user, we had to add
			 * a total of 3 nodes). We will simply destroy and recreate the UI list.
			 */
			if (this._arrayOfHierarchyItems.length > this.checkBoxList.childNodes.length) {
				var listItem = this.checkBoxList.firstChild;
				while (listItem) {
					var checkBox = listItem.props.checkBoxChild;
					if (checkBox) {
						dojo.destroy(checkBox);
					}
					listItem = listItem.nextSibling;
				}
				this._createHierarchicalItemList(this._arrayOfHierarchyItems);
			}
			
			this.checkBoxList.lastChild.scrollIntoView(false);
		},
		
		_populateTooltip: function(widget) {
			var strAry = [];
			this._mapSelectedCheckBoxes(function(checkBox) {
				strAry[checkBox.props.displayLabel] = 0;
			});
			var isFirst = true;
			var lblAry = [];
			for (lbl in strAry) {
				if (!isFirst) {
					lblAry.push("<br/>");
				} else {
					isFirst = false;
				}
				lblAry.push(lbl);
			}
			if (lblAry.length == 0) {
				lblAry.push(this.messages.noItemsSelected);
			}
			var div = document.createElement("div");
			div.innerHTML = lblAry.join("");
			widget.content = div;
		},
		
		/**
		 * Override
		 */
		init: function(reportParameterDTO) {
			// Remove the 'loading...' text.
			this._removeChildren(this.checkBoxList);
			this.inherited(arguments);
		},
		
		/**
		 * Override
		 */
		appendDefaultValues: function(reportParameterDTO, isLast, /*array of value strings*/ selectedValues) {
			if (!reportParameterDTO.labels) {
				return;
			}
			this._paramDTO = reportParameterDTO;
			
			// this is a cascading parameter. (either not a root parameter, or has child parameter)
			// do business as usual (no value filtering on large scale)
			if(!this.isRoot || this.childReportParam) {
				var labelsLen = reportParameterDTO.labels.length;
				for (var i = 0, j = this._defaultValues.length; i < labelsLen; i++) {
					var value = reportParameterDTO.values[i];
					var label = reportParameterDTO.labels[i];
					this._appendCheckBox(reportParameterDTO, value, label);
					this._defaultValues[j++] = [reportParameterDTO, value, label];
				}
			}
			else {
				this._numTotalValues = reportParameterDTO.labels.length;
				var i = 0, j = this._defaultValues.length;
				var allValues = reportParameterDTO.values;
				var allLabels = reportParameterDTO.labels;
				
				//if there are more than 50/MAX_INITIAL_DEFAULT_VALUES values
				//AND there are values to be selected/set, redo everything.
				//create the list item for the selected values first
				//the whole list will be sorted and then displayed at the end of this function. 
				if(this._numTotalValues > MAX_INITIAL_DEFAULT_VALUES && selectedValues && selectedValues.length > 0) {
	
					//reset everything
					this.uninitialize();
					j = 0;
					this._numTotalValues = reportParameterDTO.labels.length;
					
					for(i = 0; i < selectedValues.length; i++) {
						for(var p = 0; p < this._numTotalValues; p++) {
							var value = allValues[p];
							if("'" + value + "'" == selectedValues[i]) {
								var label = allLabels[p];
								var checkBox = this._appendCheckBox(reportParameterDTO, value, label);
								this._defaultValues[j++] = [reportParameterDTO, value, label];
								
								checkBox.checked = true;
								this._onCheckBoxClick({
									target: checkBox,
									fireDirtyListener: false
								});
								
								//remove this value from the collection
								allValues.splice(p, 1);
								allLabels.splice(p, 1)
								break;
							}
						}
					}
				}
				
				for (i = 0; (i < allLabels.length && j < MAX_INITIAL_DEFAULT_VALUES); i++) {
					this._appendCheckBox(reportParameterDTO, allValues[i], allLabels[i]);
					this._defaultValues[j++] = [reportParameterDTO, allValues[i], allLabels[i]];
				}
				
				// keep the remaining values in this._additionalValues if there are any
				// with the exception of special value, need to add the special check box in the first round
				this._additionalValues = [];
				this._isLast = isLast;
				if(i < allLabels.length) {
					for(; i < allLabels.length; i++) {
						if(this._isSpecialValue(allValues[i])) {
							this._appendCheckBox(reportParameterDTO, allValues[i], allLabels[i]);
							this._defaultValues[j++] = [reportParameterDTO, allValues[i], allLabels[i]];
						}
						else {
							this._additionalValues.push(
								{itemId: allValues[i],
								 label: allLabels[i]}
							);
						}
					}
				}
				//There's a chance only the special value remaining and got created as a check box.
				//There won't be any remaining to add.  So check the length to be sure.
				if(this._additionalValues.length > 0) {	
					var button = this._toolbar.getItem(0);
					if(button && button.domNode.id != ADD_VALUES_BUTTON_ID) {
						var button = this._toolbar.addButton(this, this._handleAddValuesAction, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/add.gif"), label: this.messages.addMoreValues, index: 0});
						button.domNode.id = ADD_VALUES_BUTTON_ID;
					}
				}
			}
			
			this._updateHeaderSelectedStats();
			if (!isLast) {
				return;
			}
			
			if ((reportParameterDTO.flags & this.statics.HIERARCHICAL) == this.statics.HIERARCHICAL) {
				this._createHierarchicalItemList(this._arrayOfHierarchyItems);
			} else {
				this._sortAndCreateItemList(this._arrayOfItems);
			}
			
			// enable select all/deselect all and "+" sign if it exists
			this._toolbarDiv.style.display = "block";
		},
		
		_sortAndCreateItemList: function(arr) {
			// Optimize the case of data that is not hierarchical and is sorted. Get the items from
			// arrayOfItems, sort them, and add them all at once to the listbox.
			var n = arr.length;
			if (n > 0) {
				var sortBy = this.parameterDTO.sort;
				this._removeChildren(this.checkBoxList);	// in case we already wrote out some contents.
				var sortFunc;
				switch (sortBy) {
					case this.statics.SORT_BY_VALUE_ASC:
						sortFunc = function(a,b) {
							return a.value.localeCompare(b.value);
						}
						break;
					case this.statics.SORT_BY_VALUE_DESC:
						sortFunc = function(a,b) {
							return b.value.localeCompare(a.value);
						}
						break;
					case this.statics.SORT_BY_LABEL_ASC:
						sortFunc = function(a,b) {
							return a.label.localeCompare(b.label);
						}
						break;
					case this.statics.SORT_BY_LABEL_DESC:
						sortFunc = function(a,b) {
							return b.label.localeCompare(a.label);
						}
						break;
				}
				arr.sort(sortFunc);
				
				var fragment = document.createDocumentFragment();
				for (var i = 0; i < n; ++i) {
					fragment.appendChild(arr[i].item);
				}
				this.checkBoxList.appendChild(fragment);
			}
		},
		
		/*
		 * Take the in-memory representation of the hierarchical list and put
		 * it in the UI
		 */
		_createHierarchicalItemList: function(arr) {
			var fragment = document.createDocumentFragment();
			var n = arr.length;
			for (var i = 0; i < n; ++i) {
				fragment.appendChild(arr[i]);
			}
			this.checkBoxList.appendChild(fragment);
		},
		
		/**
		 * Override
		 */
		setDefaultValues: function() {
			var defaultValue = this.parameterDTO.defaultValue;
			if (defaultValue && defaultValue.length > 0) {
				var defaultCheckBoxId = this._uuid + "_" + defaultValue;
				var defaultCheckBox = dojo.byId(defaultCheckBoxId);
				if (defaultCheckBox) {
					if (!defaultCheckBox.checked) {
						defaultCheckBox.checked = true;
						this._onCheckBoxClick({
							target: defaultCheckBox,
							reportParameterDTO: this._reportParameterDTOs[defaultCheckBoxId],
							fireDirtyListener: false
						});
					}
					return true;
				}
			}
			return false;
		},
		
		/**
		 * Override
		 */
		resetValues: function() {
			this._deselectAll();
			this.setDefaultValues();
		},
		
		// Is this method called from anywhere?  The implementation here checks all checkBoxes, regardless of the passed in values
		setSelectedValues: function(values) {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				if (!checkBox.checked) {
					checkBox.checked = true;
				}
			}));
		},
		
		getSelectedValues: function() {
			//special case to handle when the user has selected all values - to avoid passing a large number
			//of values, use special return value instead
			if (this.isRoot && !this.parameterDTO.concealed) {
				if (!this._additionalValues || this._additionalValues.length === 0) {
					//if we are displaying all possible values, are they all selected?
					if (this._numSelectedValues === this._numValues) {
						return [SELECT_ALL];
					}
				}
			}
			
			var values = [];
			this._mapSelectedCheckBoxes(function(checkBox) {
				if (!checkBox.props.isDuplicate) {
					values.push(checkBox.props.paramValue);
				}
			});			
			
			return values;
		},
		
		loadDefaults: function(reportParameterDTOs) {
			this.cascadeClear();
			if (!reportParameterDTOs) {
				var len = this.checkBoxList.childNodes.length;
				for (var i = 0; i < len; i++) {
					this._numValues++;
					var li = this.checkBoxList.childNodes[i];
					if (li.props.checkBoxChild) {
						dojo.removeClass(li, "display-none");
						this._displayGhostParents(li);
					}
				}
				dojo.removeClass(this.specialParamSpan, "display-none");
				this._updateHeaderSelectedStats();
				if (this.childReportParam) {
					this.childReportParam.loadDefaults();
				}
			} else {
				this.addValues(reportParameterDTOs);
			}
		},
		
		removeAllAndAdd: function(reportParameterDTOs) {
			this.cascadeClear();
			this.addValues(reportParameterDTOs);
		},
		
		cascadeClear: function() {
			this.clear();
			if (this.childReportParam) {
				this.childReportParam.cascadeClear();
			}
		},
		
		addValues: function(reportParameterDTOs) {
			for (var i = 0; i < reportParameterDTOs.length; i++) {
				var reportParameterDTO = reportParameterDTOs[i];
				if (reportParameterDTO) {
					this._addedReportParameterDTOs.push(reportParameterDTO);
					for (var j = 0; j < reportParameterDTO.values.length; ++j) {
						var value = reportParameterDTO.values[j];
						var cb = dojo.byId(this._uuid + "_" + value);
						if (cb) {
							if (cb.props.isSpecial) {
								if (dojo.hasClass(this.specialParamSpan, "display-none")) {
									dojo.removeClass(this.specialParamSpan, "display-none");
									this._numValues++;
								}
							} else if (dojo.hasClass(this._idToListItem[cb.id], "display-none")) {
								this._numValues++;
								this._idToListItem[cb.id].className = "";
								this._displayGhostParents(this._idToListItem[cb.id]);
							}
						}
					}
					if (this.childReportParam) {
						this.childReportParam.addValues(reportParameterDTO.children);
					}
				}
			}
			this._updateHeaderSelectedStats();
		},
		
		_displayGhostParents: function(listItem) {
			var parentListItem = listItem.props.parentListItem;
			while (parentListItem) {
				if (dojo.hasClass(parentListItem, "display-none")) {
					dojo.removeClass(parentListItem, "display-none");
					if (!parentListItem.props.checkBoxChild.disabled) {
						this._numValues++;
					}
				}
				parentListItem = parentListItem.props.parentListItem;
			}
		},
		
		setValues: function(values) {
			values = StringUtils.splitValues(values);
			
			//value of "*" indicates that all should be selected (including additional values that are not currently shown			
			if (values.length === 1 && values[0] === SELECT_ALL	) { // "all" selected
				if (this._additionalValues && this._additionalValues.length > 0) {
					//since "all" are selected, we need to add all of the additional values to the list
					this._addMoreValuesToList(this._additionalValues.slice(0)); //use slice to make a copy of the array
				}
				
				//select all values - we will not call _selectAll because that method fires dirty listeners, which we don't want
				this._mapCheckBoxes(dojo.hitch(this, function(checkBox){
					if (!checkBox.checked && !checkBox.disabled && (!dojo.hasClass(this._idToListItem[checkBox.id], "display-none") || checkBox.props.isSpecial)) {
						checkBox.checked = true;
						this._onCheckBoxClick({
							target: checkBox,
							fireDirtyListener: false,
							propagate: false
						});
					}					
				}));
			} else if(this.isRoot === true && !this.childReportParam &&  //not a cascading parameter
			   this._additionalValues && this._additionalValues.length > 0) {
				//For non-cascading parameters,
				//if there are more than 50 values, only the first MAX_INITIAL_DEFAULT_VALUES/50 are shown in the list 
				//the value to be set/selected may or may not exist in the list yet
				//We can't just append them since we want only 50 max shown unless there are more than 50 selected
				//In this case, clear the list and recreate to guarantee the selected values are included.				
				this.appendDefaultValues(this._paramDTO, this._isLast, values);
			}
			//Otherwise, do business as usual.
			else {			
				this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
					if (!checkBox.checked) {
						for (var i = 0; i < values.length; i++) {
							if (checkBox.props.paramValue == values[i]) {
								checkBox.checked = true;
								this._onCheckBoxClick({
									target: checkBox,
									fireDirtyListener: false
								});
								// Check duplicates
								this._mapCheckBoxDuplicates(checkBox, dojo.hitch(this, function(dupCheckBox) {
									dupCheckBox.checked = true;
									this._onCheckBoxClick({
										target: dupCheckBox,
										fireDirtyListener: false
									});
								}));				 
							}
						}
					}
				}));
			}
		},
		
		getSelectedLabels: function() {
			var labels = [];
			this._mapSelectedCheckBoxes(dojo.hitch(this, function(checkBox) {
				labels.push(checkBox.props.displayLabel);
			}));
			return labels;
		},
		
		clear: function() {
			this._numSelectedValues = 0;
			this._numValues = 0;
			this._addedReportParameterDTOs = [];
			this._hideAll();
			dojo.addClass(this.specialParamSpan, "display-none");
		},
		
		_handleValuesMouseOver: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.valuesMouseOver) {
				this.valuesMouseOver(this._valuesSection);
			}
		},
		
		_handleValuesMouseOut: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.valuesMouseOut) {
				this.valuesMouseOut(this._valuesSection);
			}
		},
		
		_hideAll: function() {
			this._mapCheckBoxes(dojo.hitch(this, function(checkBox) {
				var li = this._idToListItem[checkBox.id];
				if (li) {
					li.className = "display-none";
				}
				checkBox.checked = false;
			}));
		},
		
		_updateHeaderSelectedStats: function () {
			this.spanSelected.innerHTML = dojo.string.substitute(this.messages.numSelected, [this._numSelectedValues, this._numValues]);
			if(this.isRoot === true && !this.childReportParam && 
			   this._additionalValues.length > 0)
				this.spanTotalNumber.innerHTML = dojo.string.substitute(this.messages.numTotal, [this._numTotalValues]);
					
			if (this._tooltip != null) {
				this._tooltip.destroy();
			}
			this._tooltip = new jazz.ui.HoverPopup({
				around: this.spanSelected,
				createContents: dojo.hitch(this, this._populateTooltip)
			});
		},
		
		_mapCheckBoxes: function(func) {
			if (this._specialCheckBox) {
				func(this._specialCheckBox);
			}
			var listItem = this.checkBoxList.firstChild;
			while (listItem && listItem.props) {
				var checkBox = listItem.props.checkBoxChild;
				if (checkBox) {
					func(checkBox);
				}
				listItem = listItem.nextSibling;
			}
		},
		
		_mapCheckBoxDuplicates: function(checkBox, func) {
			if (checkBox.duplicateNum) {
				for (var i in this._dupCheckBoxes[checkBox.id]) {
					var dupCheckBox = this._dupCheckBoxes[checkBox.id][i];
					if (dupCheckBox) {
						func(dupCheckBox);
					}
				}
			}
		},
		
		_mapSelectedCheckBoxes: function(func) {
			var self = this;
			this._mapCheckBoxes(function(checkBox) {
				if (checkBox.checked) {
					func(checkBox);
					if (checkBox.duplicateNum) {
						self._mapCheckBoxDuplicates(checkBox, function(dupCheckBox) {
							func(dupCheckBox);
						});
					}
				}
			});
		},
		
		_onCheckBoxClick: function(event) {
			var checkBoxClicked = event ? event.target : window.event ? window.event.srcElement : null;
			var fireDirtyListener = event ? (event.fireDirtyListener == undefined ? true : event.fireDirtyListener) : true;
			var propagateToChildParam = event ? (event.propagate == undefined ? true : event.propagate) : true;
			var inc = 0;
			if (dojo.hasClass(this._idToListItem[checkBoxClicked.id], "selected")) {
				inc = -1;
				this._idToListItem[checkBoxClicked.id].className = "";
				checkBoxClicked.checked = false;
			} else {
				inc = 1;
				this._idToListItem[checkBoxClicked.id].className = "selected";
				checkBoxClicked.checked = true;
			}
			if (this.childReportParam && propagateToChildParam == true) {
				var checkBoxesClicked = [];
				this._mapSelectedCheckBoxes(dojo.hitch(this, function(checkBox){
					var dto = this._reportParameterDTOs[checkBox.id];
					if (this.isRoot || this._addedReportParameterDTOs.length == 0) {
						checkBoxesClicked.push(checkBox);
					} else {
						if (event && event.reportParameterDTO != undefined) {
							checkBoxesClicked.push(checkBoxClicked);
						} else {
							for (var i in this._addedReportParameterDTOs) {
								if (dto == this._addedReportParameterDTOs[i]) {
									checkBoxesClicked.push(checkBox);
									break;
								}
							}
						}
					}
				}));
				var reportParameterDTOsToAdd = [];
				for (var i = 0; i < checkBoxesClicked.length; i++) {
					var currCheckBox = checkBoxesClicked[i];
					var selectedReportParameterDTO = this._reportParameterDTOs[currCheckBox.id];
					var values = selectedReportParameterDTO.values;
					for (var j = 0; j < values.length; ++j) {
						if (currCheckBox.props.paramValue == "'" + values[j] + "'") {
							if (selectedReportParameterDTO.children[j]) {
								reportParameterDTOsToAdd.push(selectedReportParameterDTO.children[j]);
							}
						}
					}
				}
				
				this.childReportParam.cascadeClear();
				if (reportParameterDTOsToAdd.length == 0) {
					this._setAllOrNoneSelections();
				} else {
					this.childReportParam.addValues(reportParameterDTOsToAdd);
				}
			}
			if (checkBoxClicked.props.isDuplicate == undefined && 
				((inc == 1 && this._numSelectedValues < this._numValues) || 
				 (inc == -1 && this._numSelectedValues > 0))) {
				this._numSelectedValues += inc;
				this._updateHeaderSelectedStats();
			}
			if (fireDirtyListener == true) {
				this.onDirty();
			}
		},
		
		_isSpecialValue: function(value) {
			if(value == this.statics.MY_TEAM_AREAS || 
				value == this.statics.CURRENT_ITERATION ||
				value == this.statics.CURRENT_CONTRIBUTOR)
					return true;
			else
					return false;
		},
		
		_appendCheckBox: function (reportParameterDTO, value, label) {
			var paramName = reportParameterDTO.parameterName;
			var checkBoxId = this._uuid + "_" + value;
			var isHidden = false;
			
			var isSpecial = this._isSpecialValue(value);
//			if (!this._specialCheckBox &&
//				if(value == this.statics.MY_TEAM_AREAS || 
//				 value == this.statics.CURRENT_ITERATION ||
//				 value == this.statics.CURRENT_CONTRIBUTOR) {
//				isSpecial = true;				
//			}
			
			if (!isSpecial) {
				var sortBy = this.parameterDTO.sort;
				if ((reportParameterDTO.flags & this.statics.HIERARCHICAL) == this.statics.HIERARCHICAL) {
					if (!StringUtils.startsWith(label, "/")) {
						label = "/" + label;
					}
					var labelClean = label.substring(1);
					var parts = labelClean.split("/");
					var partsLen = parts.length;
					
					var parentListItem = null;
					var nodePath = "/"+parts[0];
					var i = 1;
					do {
						var listItem = this._findInHierarchyItems(this._uuid + "_" + nodePath + "_Li");
						if (listItem == null) {
							listItem = this._createListItem(label == nodePath ? value : nodePath, parts[i-1], nodePath, parentListItem, i-1, (i-1)*15, nodePath == label);
							listItem.props.parentListItem = parentListItem;
							this._reportParameterDTOs[listItem.props.checkBoxChild.id] = reportParameterDTO;
							isHidden = listItem.props.checkBoxChild.props.isDuplicate;
							if (!isHidden) {
								if (sortBy == this.statics.NO_SORT) {
									this.checkBoxList.appendChild(listItem);
								}
								else {
									var listItems = !parentListItem ? this._rootListItems : parentListItem._childListItems;
									listItems = this._insertListItem(listItem, parentListItem, listItems, sortBy, true);
									if (!parentListItem) {
										this._rootListItems = listItems.slice();
									} else {
										parentListItem._childListItems = listItems.slice();
									}
								}
							}
						} else if (label == nodePath) {
							// We encountered a child for this parent in a previous call, must enable parent
							listItem.props.checkBoxChild.disabled = false;
							listItem.props.labelChild.style.color = "#000000";
							listItem.onclick = dojo.hitch(this, function() {
							this._onCheckBoxClick({target: listItem.props.checkBoxChild, fireDirtyListener: false, propagate: true});});
							listItem.onmouseover = function() {
								dojo.addClass(listItem, "highlight-multiselect");
							};
							listItem.onmouseout = function() {
								dojo.removeClass(listItem, "highlight-multiselect");
							};
						}
						if (i == partsLen) {
							break;
						}
						nodePath += "/" + parts[i];
						i++;
						parentListItem = listItem;
					} while (true);
				} else {
					// For non-hierarchical sorted values, don't insert into the DOM one-by-one.
					// Put the item into arrayOfItems, and let appendDefaultValues sort and insert
					// them en masse.
					var listItem = this._createListItem(value, label, null, null, 0, 0, true);
					this._reportParameterDTOs[listItem.props.checkBoxChild.id] = reportParameterDTO;
					isHidden = listItem.props.checkBoxChild.props.isDuplicate;
					if (!isHidden) {
						if (sortBy != 0) {
							// Put the item into arrayOfItems rather than inserting one-by-one into the DOM.
							// The arrayOfItems will get inserted en masse in appendDefaultValues.
							this._arrayOfItems.push( {'item': listItem, 'value': value, 'label': label } );  
						}
						else {
							this.checkBoxList.appendChild(listItem);
						}
					}
				}				
			} else if (!this._specialCheckBox) {
				this._specialCheckBox = this._createSelectOption(value);
				this._specialCheckBox.props.displayLabel = this._localizedLabels[value];
				this._specialCheckBox.props.isSpecial = true;
				this._specialCheckBox.className = "";
				this.specialParamSpan.appendChild(this._specialCheckBox);	
				this.specialParamSpan.appendChild(document.createTextNode(this._specialCheckBox.props.displayLabel));
				this._reportParameterDTOs[this._specialCheckBox.id] = reportParameterDTO;
			}
			if (!isHidden) {
				this._numValues++;
			}
			
			return isSpecial ? this._specialCheckBox : listItem.props.checkBoxChild;
		},
		
		_findInHierarchyItems: function(id) {
			var n = this._arrayOfHierarchyItems.length;
			for (var i = 0; i < n; ++i) {
				var item = this._arrayOfHierarchyItems[i];
				if (id == item.id) {
					return item;
				}
			}
		},
		
		_createSelectOption: function(value, displayLabel) {
			var checkBox = null;
			if (displayLabel) {
				// Inner select option
				checkBox = document.createElement("div");
			} else {
				// Outer special select option - such as 'Current Iteration'
				checkBox = document.createElement("input");
				checkBox.onclick = dojo.hitch(this, this._onCheckBoxClick);
			}
			checkBox.type = "checkbox";
			var props = {};
			checkBox.props = props;
			props.paramValue = "'" + value + "'";
			props.displayLabel = displayLabel;
			var checkBoxId = this._uuid + "_" + value;
			var currCheckBoxLi = this._idToListItem[checkBoxId];
			var currCheckBox;
			if (currCheckBoxLi) {
				currCheckBox = currCheckBoxLi.props.checkBoxChild;
			}
			if (currCheckBox) {
				if (!this._dupCheckBoxes[checkBoxId]) {
					this._dupCheckBoxes[checkBoxId] = [];
				}
				this._dupCheckBoxes[checkBoxId].push(checkBox);
				if (!currCheckBox.duplicateNum) {
					currCheckBox.duplicateNum = 0;
				}
				currCheckBox.duplicateNum++;
				checkBoxId = checkBoxId + "_" + currCheckBox.duplicateNum;
				props.isDuplicate = true;
			}
			checkBox.id = checkBoxId;
			if (!displayLabel) {
				this._idToListItem[checkBox.id] = document.createElement("div"); // placeholder
			}
			return checkBox;	
		},
		
		_createListItem: function(value, displayLabel, nodePath, parentListItem, nodeLevel, paddingLeft, isEnabled) {
			var checkBox = this._createSelectOption(value, displayLabel);
			var li = document.createElement("li");
			this._idToListItem[checkBox.id] = li;
			li.id = this._uuid + "_" + nodePath + "_Li";
			var props = {};
			li.props = props;
			props.checkBoxChild = checkBox;
			props.paramValue = value;
			props.nodePath = nodePath;
			props.displayLabel = displayLabel;
			props.nodeLevel = nodeLevel;
			props.parentListItem = parentListItem;
			if (checkBox.props.isDuplicate) {
				li.className =  "display-none";
			}
			li.style.width = "100%";
			var labelElem = dojo.doc.createElement("label");
			props.labelChild = labelElem;
			li.appendChild(labelElem);
			labelElem.style.paddingLeft = paddingLeft + "px";
			if (!isEnabled) {
				checkBox.disabled = true;
				labelElem.style.color = "#999999";
			} else {
				li.onclick = dojo.hitch(this, function() {
					this._onCheckBoxClick({target: checkBox, fireDirtyListener: true, propagate: true});
				});
				li.onmouseover = function() {
					dojo.addClass(li, "highlight-multiselect");
				};
				li.onmouseout = function() {
					dojo.removeClass(li, "highlight-multiselect");
				};
			}
			labelElem.appendChild(checkBox);
			labelElem.appendChild(document.createTextNode(displayLabel));
			return li;
		},
		
		_insertListItem: function(newListItem, parentListItem, listItems, sortBy, isHierarchy) {
			if (!listItems || listItems.length == 0) {
				// Just insert into parent, none to compare against
				if (parentListItem != null) {
					var index = this._indexOf(this._arrayOfHierarchyItems, this._getNextSibling(parentListItem));
					if (index = -1) {
						index = this._indexOf(this._arrayOfHierarchyItems, parentListItem) + 1;
					}
					this._arrayOfHierarchyItems.splice(index, 0, newListItem);
				} else {
					this._arrayOfHierarchyItems.push(newListItem);
				}
				if (isHierarchy) {
					if (!listItems) {
						listItems = [];
					}
					listItems.push(newListItem);
				}
			} else {
				var lastSibling = null;
				var low = 0;
				var high = listItems.length;
				var mid = 0;
				while(low < high) {
					mid = Math.floor((low + high) / 2);
					var li = listItems[mid];
					switch (sortBy) {
						case this.statics.SORT_BY_VALUE_ASC:
							if (newListItem.props.paramValue.localeCompare(li.props.paramValue) < 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_VALUE_DESC:
							if (newListItem.props.paramValue.localeCompare(li.props.paramValue) > 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_LABEL_ASC:
							if (newListItem.props.displayLabel.localeCompare(li.props.displayLabel) < 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
						case this.statics.SORT_BY_LABEL_DESC:
							if (newListItem.props.displayLabel.localeCompare(li.props.displayLabel) > 0) {
								high = mid;
							} else {
								low = mid + 1;
							}
							break;
					}
					lastSibling = li;
				}
				if (low == 0) {
					this._arrayOfHierarchyItems.splice(this._indexOf(this._arrayOfHierarchyItems, lastSibling), 0, newListItem);
				} else {
					var li = listItems[low - 1];
					var nextSibling = this._getNextSibling(li);
					if (nextSibling && nextSibling.props.nodeLevel == newListItem.props.nodeLevel) {
						this._arrayOfHierarchyItems.splice(this._indexOf(this._arrayOfHierarchyItems, nextSibling), 0, newListItem);
					} else {
						li = this._getNextSibling(li);
						while (li && li.props.nodeLevel > newListItem.props.nodeLevel) {
							li = this._getNextSibling(li);
						}
						if (li) {
							this._arrayOfHierarchyItems.splice(this._indexOf(this._arrayOfHierarchyItems, li), 0, newListItem);
						} else {
							this._arrayOfHierarchyItems.push(newListItem);
						}
					}
				}
				if (isHierarchy) {
					listItems.splice(low,0, newListItem);
				}
			}
			return listItems;
		},
		
		_indexOf: function(array, item) {
			if(!Array.indexOf){
		        for(var i=0; i<array.length; i++){
		            if(array[i] == item){
		                return i;
		            }
		        }
		        return -1;
			} else {
				return array.indexOf(item);
			}
		},
		
		_getNextSibling: function(li) {
			var index = this._indexOf(this._arrayOfHierarchyItems, li);
			if (index < this._arrayOfHierarchyItems.length - 1) {
				return this._arrayOfHierarchyItems[++index];
			} else {
				return undefined;
			}
		},
		
		_setAllOrNoneSelections: function() {
			if (this.childReportParam) {
				this.childReportParam.cascadeClear();
				var dtos = null;
				var reportParameterDTOsToAdd = [];
				if (this.isRoot) {
					var values = this._paramDTO.values;
					for (var j = 0; j < values.length; ++j) {
						if (this._paramDTO.children[j]) {
							reportParameterDTOsToAdd.push(this._paramDTO.children[j]);
						}
					}
				} else {
					if (this._addedReportParameterDTOs.length == 0) {
						dtos = this._reportParameterDTOs;
					}
					else {
						dtos = this._addedReportParameterDTOs;
					}
					for (var k in dtos) {
						var dto = dtos[k];
						if (!dto.children) {
							continue;
						}
						var vals = dto.values;
						for (var j = 0; j < vals.length; ++j) {
							if (dto.children[j]) {
								reportParameterDTOsToAdd.push(dto.children[j]);
							}
						}
					}
				}
				this.childReportParam.addValues(reportParameterDTOsToAdd);
			}
		},
		
		remoteError: function(error) {
			alert(error.message);
		}
		
	});
	
})();
