/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader");

dojo.require("dijit.ToolbarSeparator");

dojo.require("dojox.xml.parser"); // dojo.dom.removeChildren
dojo.require("dojo.date");
dojo.require("dojo.fx");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.mvvm.model.WorkItemWorkingCopy"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.module.WindowContext");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics"); //$NON-NLS-1$
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui2.internal.action.ActionFactory");
dojo.require("com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor");
dojo.require("com.ibm.team.rtc.foundation.web.ui.widgets.StatusIndicator");
dojo.require("com.ibm.team.rtc.foundation.web.ui.util.Sprites"); //$NON-NLS-1$
dojo.require("com.ibm.team.rtc.foundation.web.ui.util.sprite.ImageSource");
dojo.require("com.ibm.team.rtc.foundation.web.text.XMLString");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.actions.createcopy.CreateWorkItemCopyAction");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var Toolbar						= jazz.ui.Toolbar;
var Button						= jazz.ui.toolbar.Button;
var checkNotNull				= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler		= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WorkItemUtil				= com.ibm.team.workitem.web.client.util;
var EditorUtil					= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var LayoutUtil					= com.ibm.team.workitem.web.ui.internal.view.layout.LayoutUtil;
var ImageClient					= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemClient				= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WorkItemExtensionRegistry	= com.ibm.team.workitem.web.client.internal.WorkItemExtensionRegistry;
var ImagePool					= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
var ImageSource					= com.ibm.team.rtc.foundation.web.ui.util.sprite.ImageSource;
var WidgetUtil					= com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ActionFactory				= com.ibm.team.workitem.web.ui2.internal.action.ActionFactory;
var CreateWorkItemCopyAction	= com.ibm.team.workitem.web.ui.internal.view.editor.actions.createcopy.CreateWorkItemCopyAction;
var WorkItemWorkingCopy			= com.ibm.team.workitem.web.mvvm.model.WorkItemWorkingCopy;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest						= com.ibm.team.repository.web.client.session.isGuest;
var Sprites						= com.ibm.team.rtc.foundation.web.ui.util.Sprites;
var XMLString					= com.ibm.team.rtc.foundation.web.text.XMLString;

var SUBSCRIBERS_ID= "internalSubscriptions";

var COMPATIBLE_TYPES		= [ ["defect", "task", "enhancement"] ];
var BUNDLE_ID				= "com.ibm.team.workitem.web";

//Paths to the icons that switch as state changes
var FIND_DUPLICATES_ENABLED=  "ui/internal/view/editor/images/find-duplicates.gif";  // $NON-NLS-1$
var SUBSCRIBE= "/ui/internal/view/editor/images/subscribe.gif"; // $NON-NLS-1$
var UNSUBSCRIBE= "/ui/internal/view/editor/images/unsubscribe.gif"; // $NON-NLS-1$

var MAX_MESSAGE_LENGTH= 500;

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader",
	[com.ibm.team.workitem.web.ui2.internal.widget.AbstractEditor],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditorHeader.html"),
		
		_lastSaveTime: null,
		errorHandlingMap: {
			ALERT: function(self, message) {
				alert(message.message + "\n\n" + message.details);
			},
			ADD_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating= true;
				self.isValidationg= false;
				self.parentController.notifyLinkManagerToAddDuplicateLink();
				self.parentController.showLinksTab();
			},
			SET_RESOLUTION_TO_DUPLICATE_ACTION: function(self) {
				self.isValidating= true;
				self.parentController.showLinksTab();
				self.isValidationg= false;
				alert(self.messages.msgResolutionSet);
			},
			REMOVE_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating= true;
				self.parentController.showLinksTab();
				self.isValidationg= false;
				self.parentController.notifyLinkManagerToRemoveDuplicateLink();
			},
			SHOW_INFOS_ACTION: function(self) {
				var infos= self._currentInfos;
				if (infos && infos.length> 0) {
					var msg= "";
					for (var i=0;i<infos.length;i++) {
						msg += infos[i].summary + ".\n";
					}
					alert(msg);
				}
			}
		},
		constructor: function() {
			this.messages= dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

			this.titleSubscribeMe= this.messages.titleSubscribeMe;
			this.titleUnsubscribeMe= this.messages.titleUnsubscribeMe;
			this.titleMoveCopy= this.messages.titleMoveCopy;
			this.titleFindPotentialDuplicates= this.messages.titleFindPotentialDuplicates;
			this.titleCancelWorkItem= this.messages.titleCancelWorkItem;
			this.titleRefreshWorkItem= this.messages.titleRefreshWorkItem;
			this.titleRemoveWorkItem= this.messages.titleRemoveWorkItem;
			this.buttonCancel= this.messages.buttonCancel;
			this.buttonSave= this.messages.buttonSave;
			this.progressMessage= this.messages.busyLoading;
			this.labelSummarySafari= this.messages.labelSummary;		
		
		},
	
		
		// populate the template with data, connect events, and show initial
		// child widget
		postCreate: function() {
			checkNotNull(this.parentController, "this.parentController");
			checkNotNull(this.editorUtil, "this.editorUtil");
						
			this.isHeader= true;
			this._titleIconLinkNode.appendChild(Sprites.create(ImagePool.WORKITEM_EDITOR, {className: "EditorIcon", draggable: "false"}));
			
			this._childWidgets= [];
			this.statusIndicator= new com.ibm.team.rtc.foundation.web.ui.widgets.StatusIndicator({}, this._summarySafariStatusIcon);
			var reqMark= new com.ibm.team.workitem.web.ui.internal.view.layout.RequiredPropertyMarkWidget({
					workingCopy: this.parentController.workingCopy,
					 attributeId: "summary"
			});
			this._childWidgets.push(reqMark);
			this.summaryRequiredMarker.appendChild(reqMark.domNode);
			// Defect 66867 Truncated summary (pl locale this.labelSummary= "Podsumowanie: ";)
			if (this.labelSummarySafari.length > 12) {
				this.summaryLabelDIV.className= "SummaryLabelVeryLong";
			}
			else 
				if (this.labelSummarySafari.length > 9) {
					this.summaryLabelDIV.className= "SummaryLabelLong";
				}
			this.headerSummary= this.headerSummarySafari;
			this.inputDiv= this.inputDivSafari;
			
			this._isInitialized= false;
			this._updateTypeIcon();
			
			this.layoutHeader();
			this._resizeEvent= jazz.util.ResizeNotifier.register({context: this, callback: this._onResize});

			this._handles= [ dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_CHANGED, dojo.hitch(this, this._handleWorkItemChanged)) ];
			this._handles.push(dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_STORED, dojo.hitch(this, this._handleWorkItemChanged)));
			
			this._toolbar= new Toolbar();
			this._childWidgets.push(this._toolbar);
			this._simpleToolbarContainer.appendChild(this._toolbar.domNode);

			this._actions= [];
			var extensionActions= WorkItemExtensionRegistry.getContributedEditorToolbarActions();
			if (extensionActions.length > 0) {
				for (var i= 0; i < extensionActions.length; i++) {
					var actionSpec= extensionActions[i];
					if (actionSpec.action) {
						var actionParms= {actionSpec: actionSpec, workingCopy: this.workingCopy};
						var action= ActionFactory.getAction(actionParms);
						if (!action) {
							if(window.location.href.indexOf("_ajax-modules") == -1) { // $NON-NLS-1$ Log error if not testing			
								console.debug("Missing action: " + actionSpec.action);
							}
							continue;
						}
						var isEnabled= action.isEnabled(actionParms);
						if (action.isVisible(actionParms)) {
							var isInitiallyVisible= action.isInitiallyVisible !== false;
							var button= new Button({
									context: action,
									title: actionSpec.label, 
									imgClass: Sprites.cssClassName(action.getImageSource()), 
									alt: actionSpec.alt, 
									enabled: isEnabled, 
									visible: isInitiallyVisible,
									onClick: dojo.hitch(/*context*/ action, action._doRun)
							});
							if (dojo.isFunction(action.initialize)) {
								action.initialize(dojo.hitch({b:button, a:action}, function() {
									if (this.b.domNode) {
										this.b.set('visible', this.a.isVisible());
										this.b.set('enabled', this.a.isEnabled());
									}
								}));
							}
							this._toolbar.addChild(button);
						}
						this._actions.push(action);
					}
				}
			
				// Add a separator between the extensions and default actions
				this._toolbar.addChild(new dijit.ToolbarSeparator());
			}

			this._findDuplicatesButton= new Button({
					title: this.titleFindPotentialDuplicates,
					enabled: true, 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, FIND_DUPLICATES_ENABLED)),
					onClick: dojo.hitch(this, this._handleFindDuplicatesClick)
			});
			this._subscribeSelfButton= new Button({
					title: this.titleSubscribeMe,
					enabled: true, 
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, SUBSCRIBE)),
					onClick: dojo.hitch(this, this._handleSubscribeSelfClick)
			});

			this._subscribeSelfButtonEnabled= true;
			
			this._moveCopyButton= new Button({
					title: this.titleMoveCopy,
					enabled: true, 
					visible: false,
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/editor/images/move-copy.gif")),
					onClick: dojo.hitch(this, this._handleMoveCopyClick)
			});
			this._createWorkItemCopyAction= new CreateWorkItemCopyAction(this.workingCopy.getWorkingCopy(), {
					enabled: true, 
					visible: false,
					onWorkItemCopied: function(workingCopy) {
						location.replace(workingCopy._proxy.url);
					}
			});
			this._refreshButton= new Button({
					title: this.titleRefreshWorkItem,
					enabled: true, 
					visible: true,
					imgClass: Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, "ui/internal/view/editor/images/refresh.gif")),
					onClick: dojo.hitch(this, this._handleRefreshClick) 
			});

			this._toolbar.addChild(this._findDuplicatesButton);
			this._toolbar.addChild(this._createWorkItemCopyAction.getActionButton());
			this._toolbar.addChild(new dijit.ToolbarSeparator());

			this._toolbar.addChild(this._subscribeSelfButton);
			this._toolbar.addChild(this._moveCopyButton);
			this._toolbar.addChild(this._refreshButton);
			
			var helpContextId= com.ibm.team.workitem.web.ui.internal.view.common.ContextHelpTopics.WORK_ITEM_EDITOR_PAGE_CS_HELP;
			this._pageHelp= new jazz.app.ua.PageHoverHelp({uaKey: helpContextId});
			this._pageHelp.placeAt(this._contextHelp);
			this._handles.push(dojo.subscribe(com.ibm.team.workitem.web.ui.internal.page.EventTopics.WORKITEM_READ_ONLY_PROPERTIES_CHANGED, this, dojo.hitch(this, this._handleReadOnlyPropertyChanged)));
		},
		
		resetChildWidgets: function(isLayout) {
			if (!isLayout) {
				this.inherited(arguments);
				return;
			}
			/* 
			 * For a reset from the layout, skip destroying the summary and the toolbar
			 */
			for (var key in this._childWidgets) {
				/*
				 * Summary and toolbar should not be destroyed
				 */
				if (this._childWidgets[key] && (this._childWidgets[key].attributeId == "summary" || this._childWidgets[key].declaredClass == "jazz.ui.Toolbar")) {
					continue; 					
				}

				if (this._childWidgets[key] && this._childWidgets[key].destroyWidget) {
					this._childWidgets[key].destroyWidget();
				} else if (this._childWidgets[key] && this._childWidgets[key].destroy) {
					this._childWidgets[key].destroy();
				}
				this._childWidgets[key]= null;
	//95375			delete this._childWidgets[key];
			}
			this._childWidgets= [];
			 
		},
		getSummaryValue: function() {
			if (this.summary) {
				var value= this.summary.getValue();
				return dojo.isFunction(value.getXMLText) ? value.getXMLText() : value;
			}else{
				return this._origSummaryVal;
			}
		},
		
		setSummaryValue: function(val) {
			if (this.summary) {
				this.summary.setValue(XMLString.createFromXMLText(val));
			}else{
				this._origSummaryVal= val;
			}
		},
		
		getSummaryNode: function() {
			if (this.summary) {
				return this.summary.domNode;
			}
			return null;
		},
		
		focusSummary: function() {
			if (this.summary && !this.summary.focused) {
				this.summary.focus();
			}
		},
		_handleReadOnlyPropertyChanged: function(parms) {
			if (!this._subscribeSelfButton || !this._subscribeSelfButtonEnabled) return;
			var isSubscribersDynamicReadOnlyValue= parms.workingCopy.isAttributeDynamicallyReadOnly({ attributeId:  "internalSubscriptions" });
			if (isSubscribersDynamicReadOnlyValue != null) {			
				this._subscribeSelfButton.set('enabled', !isSubscribersDynamicReadOnlyValue);
			}
		},
		_handleWorkItemChanged: function(workingCopy) {
			if (workingCopy === this.parentController.workingCopy) {
				var buttons= this._toolbar.getChildren();
				for (var idx= 0; idx < buttons.length; idx++) {
					var cur= buttons[idx];
					if (cur.context) {
						cur.set('enabled', cur.context.isEnabled(workingCopy));
					}
					
				}
				this.disableSave(/*isDisabled*/!workingCopy.isChanged());
				this.updateSubscribeSelf();
			} 
		},
		destroy: function() {
			jazz.util.ResizeNotifier.unregister(this._resizeEvent);
			for (var i=0;i<this._handles.length;i++) {
				dojo.unsubscribe(this._handles[i]);
			}
			// this._childWidgets are destroyed by inherited AbstractEditor protocol
			this._summaryNameValueLayout= null;
			if (this._pageHelp) this._pageHelp.destroy(false);
			this._pageHelp= false;
			if (this.statusIndicator) {
				this.statusIndicator.destroy();
				this.statusIndicator= null;
			}
			
			for (var actionIndex in this._actions) {
				this._actions[actionIndex].destroy();
			}
			this._actions= [];
			
			this.inherited(arguments);
		},
		_onResize: function() {
			var summaryNode= this.getSummaryNode();
			if (!summaryNode)
				return;
				
			var width= dojo.coords(summaryNode).w;
			if (width> 285) {
				dojo.removeClass(this.headerSummary, "SummaryWrapperWide");
				dojo.addClass(this.headerSummary, "SummaryWrapper");
			} else {
				dojo.removeClass(this.headerSummary, "SummaryWrapper");
				dojo.addClass(this.headerSummary, "SummaryWrapperWide");
			}
		},
		layoutHeader: function() {
			
			this.workingCopy= this.parentController.workingCopy;
			if (!this.workingCopy) return; // not yet initialized

			if (this.workingCopy.workItemSpec !== null && this.workingCopy.workItemSpec.presentationProps !== null) {
				var listDiv= document.createElement("div");
				var pages= this.workingCopy.workItemSpec.presentationProps.pages;
				if (!pages || pages.length === 0) 
					return;
				var headerPage;
				for (var i= 0; i < pages.length; i++) {
					if (pages[i].layout == "internalHeader") { // $NON-NLS-1
						headerPage= pages[i];
						break;
					}
				}
				this.headerElement= null; //first element in the header
				this.resetChildWidgets(/* isLayout */ true);
				dojox.xml.parser.removeChildren(this.headerLeft);
				dojox.xml.parser.removeChildren(this.headerRight);
				this.headerSummary.style.display="none";
				if (headerPage && headerPage.sections && headerPage.sections.length > 0) {
					var presentations= headerPage.sections[0].presentations;
					var isLeft= false;
					var primaryPresentation = this._getPrimaryHeaderPresentation(presentations);
					for (var i= 0; i < presentations.length; i++) {
						var widgetDiv= document.createElement("div");
						isLeft = false;
						if (presentations[i].widget.indexOf("SeparatorPart") != -1)
							continue;
						if (!this.headerElement) {
							if (primaryPresentation == null || presentations[i] == primaryPresentation) {
								this._addPrimaryHeaderElement(presentations[i], widgetDiv);
								continue;
							} else {
								isLeft = true;
							}
						} else if (presentations[i].attributeId == "summary") {
							if (this.summary) {
								this.summary.destroy();
								this.summary= null;
							}
							var values= this._getPresentationValues(presentations[i]);
							var possibleValues= this._getPresentationPossibleValues(presentations[i], false) || [];
							presentations[i].bidiSupport= true;
							this.summary= this.getWidget(this.workingCopy.object, presentations[i], widgetDiv, values, possibleValues, {webHeight: "1.2em"});
						}
						
						var targetDiv= isLeft ? this.headerLeft : this.headerRight;
						this._layoutPresentation(presentations[i], widgetDiv, null, {webHeight: "1.2em"} );
						targetDiv.appendChild(widgetDiv);
					}
				} 
			}
		},
		_getPrimaryHeaderPresentation: function(presentations) {
			for (var i= 0; i < presentations.length; i++) {
				if (this._isPrimaryHeaderPresentation(presentations[i])) {
					return presentations[i];
				}
			}
			return null;
		},
		_isPrimaryHeaderPresentation: function(presentation) {
			return presentation.presentationId == "com.ibm.team.workitem.kind.smallhtml";
		},
		_addPrimaryHeaderElement: function(presentation, widgetDiv) {
			this.headerSummary.style.display="";
			if (presentation.attributeLabel) {
				if (this._summaryNameValueLayout) {
					this._summaryNameValueLayout.setLabel({
						labelText: presentations[i].attributeLabel,
						presentationDescription: presentation.presentationDescription });
				} else {
					var labelText= presentation.attributeLabel || "";
					if (labelText && labelText.indexOf(":") == -1 && labelText.indexOf(String.fromCharCode(65306)) == -1) 
							labelText= dojo.string.substitute(this.messages.labelValue, [labelText]);
					dojox.xml.parser.replaceChildren(this.safariSummaryLabelSpan,	document.createTextNode(labelText));
					if(presentation.presentationDescription != null){
						this.safariSummaryLabelSpan.title= presentation.presentationDescription;	
					}
				}
				var values= this._getPresentationValues(presentation);
				var possibleValues= this._getPresentationPossibleValues(presentation, false) || [];
				presentation.bidiSupport= true;
				
				if (presentation.attributeId == "summary") {
					this.summary= this.getWidget(this.workingCopy.object, presentation, widgetDiv, values, possibleValues, {webHeight: "1.2em"});
					this.headerElement= (this.headerElement == null) ? this.summary : this.headerElement;
					if (this.summary) {
						if (this._origSummaryVal) {
							this.setSummaryValue(this._origSummaryVal);
						}
						this.inputDiv.appendChild(this.summary.domNode);
					}
				} else {
					this.headerElement= this.getWidget(this.workingCopy.object, presentation, widgetDiv, values, possibleValues, {webHeight: "1.2em"});
					this.inputDiv.appendChild(this.headerElement.domNode);
					}
			} else {
				dojo.destroy(this.safariSummaryLabelSpan); // no need for label if element does not have an attributeLabel
				dojo.destroy(this.summaryRequiredMarker); // no need to set required attribute for non-attribute based presentations
				this.headerElement= widgetDiv;
				this._layoutPresentation(presentation, widgetDiv, null, {webHeight: "1.2em"} );
				this.headerLeft.appendChild(widgetDiv);
				this.inputDiv.appendChild(this.headerLeft);
			}
		},
		_layoutPresentation: function(pres, attrDiv, attrClass, modifiers) {
			var valueDiv= document.createElement("div");
			var values= this._getPresentationValues(pres);
			var possibleValues= this._getPresentationPossibleValues(pres);
			var widgetDiv= document.createElement("div");
			var widget= this.getWidget(this.workingCopy.object, pres, widgetDiv, values, possibleValues, modifiers);		
			var properties= LayoutUtil.getPropertyMap(pres);
			attrDiv.className= "attr-container-narrow";
			var labelDiv= document.createElement("div");
			if (properties.labelVisible != "false" && dojo.isString(pres.attributeLabel)) {
				attrDiv.className= "attr-container";
				labelDiv.appendChild(document.createTextNode(dojo.string.substitute(this.messages.labelValue, [pres.attributeLabel])));
				if(pres.presentationDescription != null){
					labelDiv.title= pres.presentationDescription;	
				}				
			}
			labelDiv.className= "attr-label";
			var requiredMarkerSpan= document.createElement("SPAN");
			var isRequired= this._isRequired(pres.attributeId);
			
			requiredMarkerSpan.className= "RequiredMarker";
			requiredMarkerSpan.innerHTML= this.messages.labelRequired; // "* " innerHTML OK 
			requiredMarkerSpan.title= this.messages.titleRequiredMarker;
			if (isRequired) {
				requiredMarkerSpan.style.display= "inline";
			}
			else {
				requiredMarkerSpan.style.display= "none";
			}
			labelDiv.appendChild(requiredMarkerSpan);
			
			attrDiv.appendChild(labelDiv);
			
			var width;
			if (widget.getMinimumWidth) {
			 	width= widget.getMinimumWidth();				
			}

			valueDiv.className= attrClass || "attr-value";
			valueDiv.appendChild(widget.domNode);
			if (width) {
				attrDiv.style.width= width;
			}
			attrDiv.appendChild(valueDiv);
		},
		_isRequired: function(attributeId) {
			var isRequired= this.workingCopy.getIsRequired(attributeId);
			return isRequired;
		},
		_getPresentationValues: function (pres) {
			var attrId= pres.attributeId;
			var value= {};
			if (this.workingCopy.storedObject && this.workingCopy.storedObject.attributes) {
				value= this.workingCopy.storedObject.attributes[attrId];
			}
			return [ value ];
		},
		_getPresentationPossibleValues: function (pres) {
			var uiItems= this.workingCopy.getEditPropsValuesForAttribute(pres.attributeId);
			return uiItems ? uiItems : [];
		},
		_handleDirtyingEvent: function() {
			this.parentController.markDirty();
		},
		_handleWorkingCopySummaryChange: function(parms) {
			if (this.getSummaryValue() != parms.attributeValue) {
				this.setSummaryValue(parms.attributeValue);
			}
			this.parentController.markDirty();
			if (this.statusIndicatoer && parms.validationStatus)
				this.statusIndicator.setStatus(parms.validationStatus);
		},
		_getAttributeValue: function() {
			return this.parentController.workingCopy.getValue({ path: ["attributes", "summary", "content"] });			
		},
		_setAttributeValue: function(value, isInit, isQuiet, notifyEvenIfUnchanged) {
			this.parentController.workingCopy.setValue({
				caller: this,
				path: ["attributes", "summary", "content"],
				savedAttributePath: ["attributes", "summary", "content"],
				attributeId: "summary",
				isInitialization: isInit,
				forceNotify: notifyEvenIfUnchanged,
				isQuiet: isQuiet,
				value: value
			});
			if (this.statusIndicator) {				
				var status= this.workingCopy._validate("summary");
				this.statusIndicator.setStatus(status);
				// this.statusIndicator.test();
			}
		},
		markBusy: function(text) {
			if (!text) text= this.messages.busyWorking;
			// show the loading indicator
			this._showTypeAndId(text, "", true);
		},
		focus: function(doResize) {
			if (doResize !== false) this._onResize();
			if (this.parentController && this.parentController.workingCopy && this.parentController.workingCopy.isNewWorkItem()) {
				setTimeout(dojo.hitch(this, function() {
					// In IE, BODY is focused after focusSummary somehow, so call focusSummary after body is focused 
					if (dojo.isIE) {
						var conn= this.connect(dojo.body(), "onfocus", dojo.hitch(this, function() {
							this.focusSummary();
							this.disconnect(conn);
						}));
					} else {
						this.focusSummary();
					}
				}), 0);
			}
		},
		// Reset is only called from WorkItemEditor and is always followed by a call to bindWorkItem
		reset: function(isTypeChange) {
			if (isTypeChange) {
				this.layoutHeader(); // rebuild from scratch;
			}

			if (this.parentController.workingCopy.isInTypeChange) {
				return;
			}
			
			// unbind work item data (this._typeName is cleared by this.markBusy)
			if (isGuest()) {
				if (this.getSummaryNode())
					dojo.addClass(this.getSummaryNode(), "GuestField");
				this._subscribeSelfButton.set('visible', false);
			}
			
			this._findDuplicatesButton.set('enabled', false);

			this.clearValidationMessage(true);	
			var isChanged= false;
			if (this.parentController.workingCopy) {
				isChanged=this.parentController.workingCopy.isChanged();				
			} 
			
			this.disableSave(/*isDisabled*/!isChanged);
			// reset internal state
//			this.summary.__jazzChanged= false;
			
			if (isTypeChange) {
				this.focusSummary();
			}
		},

		setHeaderToolbarState: function(isNew) {
			if (isNew) {
				this.cancelCmd.style.display= "";
				this._subscribeSelfButton.set('visible', false);
				this._refreshButton.set('visible', false);
			} else {
				this.cancelCmd.style.display= "none";
				this._subscribeSelfButton.set('visible', true);
				this._refreshButton.set('visible', true);
			}
			
		},
		_showTypeAndId: function(typeName, idLabel, isLoading) {
				
			    if (isLoading) {
					this._contextHelp.style.display= "none";
					dojox.xml.parser.removeChildren(this.progressMsg);
					this.progressMsg.appendChild(document.createTextNode(typeName));
					this.typeIcon.style.visibility= "hidden";
					this._titleTextNode.style.display= "none";
					this.progressNode.style.display= "";
				}
				else {
					this._contextHelp.style.display= "";
					dojo.removeClass(this.typeIcon, "LoadingIcon");
					dojox.xml.parser.removeChildren(this._titleTextNode);
					this._titleTextNode.appendChild(document.createTextNode(typeName + " " + idLabel));
					if (idLabel && this._titleTextNode.isDirty) {
						idLabel= idLabel.replace(/ \*/, "") + " *";
					}
					this.disableSave(this.saveCmd.disabled);
					
					this.typeIcon.style.visibility= "visible";
					this._titleTextNode.style.display= "";
					this.progressNode.style.display= "none";
				}

		},		

		// bind work item data to the editor header (each sub-tab is responsible
		// for binding its own data)
		bindWorkItem: function(workItem, retainedState, isTypeChange, newTypeId) {
			this.parentController.workingCopy.removeAllListeners({listener:this});
			this.parentController.workingCopy.addListener({
				path: ["attributes", "summary", "content"],
				event: "onchange",
				listener: this,
				functionName: "_handleWorkingCopySummaryChange"
			});			


			// save a reference to the work item
			this._workItem= workItem;
			this._hasStickyMessage= false;
			this.isValidating= false;
			this._isAutomaticallySettingResolutionToDuplicate= false;
			this.clearValidationMessage();
			
			if (!isTypeChange) {
				this._updatedTypeName= null;
				this._updatedTypeIconSrc= null;
			}

			if (this._workItem) {
				this._titleIconLinkNode.setAttribute("href", this._workItem.locationUri);
				this._titleIconLinkNode.setAttribute("title", this._workItem.locationUri);
			}
			
			// enable or disable controls based on work item state
			this.updateSubscribeSelf();

			this._disableRefresh(false); // gets enabled as "reset" for new work items			
			
			this.showWorkItemIdAndType(isTypeChange);
			
			var isCompatibleTypeChange= true;	
			if (isTypeChange) {
				var isCompatibleTypeChange= false;
				if (retainedState.summary) {
//					this.summary.__jazzChanged= true;
					this.setSummaryValue(retainedState.summary);
				} else {
					this.setSummaryValue(this._getAttributeValue());
				}
				
				/*
				 * If it is a type change, the actions and resolutions
				 * can potentially be no longer valid.
				 * 
				 * We will assume that changes among defect,task and enhancement 
				 * are compatible and leave the actions and resolution enabled,
				 * but for changes away from or into these, we will disable
				 * the actions and resolution.
				 */
				var priorType= this._workItem.attributes.workItemType.id;
		
				for (var i=0;i<COMPATIBLE_TYPES.length;i++) {
					var typeSet= COMPATIBLE_TYPES[i];
					var found= false;
					for (var j=0;j<typeSet.length;j++) {
						if (typeSet[j]==priorType) {
							found= true;
							break;
						}
					}
					if (found) {
						for (var j=0;j<typeSet.length;j++) {
							if (typeSet[j]==newTypeId) {
								isCompatibleTypeChange= true;
							}
						}
					}
					if (isCompatibleTypeChange) break;					
				}
			} else {
				var isNew= this.parentController.workingCopy.isNewWorkItem();
				if (this._isInitialized || !isNew || this.getSummaryValue() === "") {
					var content= this.parentController.workingCopy.getValue({path: "attributes.summary.content"}) || "";
					this.setSummaryValue(content);
					for (var key in this._childWidgets) {
						var attributeModel = this.workingCopy.getWorkingCopy().getAttributeModel(key);
						if (attributeModel != null) {
							attributeModel.populateValues();
						}
					}
				} else {
					if (isNew && this.getSummaryValue() !== "") {
						this.markDirty();
					}
				}
			}
			this._isInitialized= true;

			return isCompatibleTypeChange;
		},
		endOperation: function() {
			this.showWorkItemIdAndType();
		},
		showWorkItemIdAndType: function(isTypeChange) {
			// bind work item data to UI controls
			var typeName= this._workItem.attributes.workItemType.label;
			if (isTypeChange && this._updatedTypeName) {
				typeName= this._updatedTypeName;
				this.layoutHeader();
			}
			var idLabel= this.parentController.workingCopy.getIdLabel();

			this._showTypeAndId(typeName, idLabel, /* isLoading */ false);			
	},
		// @deprecated
		bindOptionalValues: function(valMap, retainedState, isTypeChange, isCompatibleTypeChange) {

		},
		markFullyLoaded: function() {
			
			// If the type was updated, use that icon instead of the work item's
			if (this._updatedTypeIconSrc) {
				this._updateTypeIcon(this._updatedTypeIconSrc);
				return;
			}
			if (this._workItem.attributes.workItemType.iconUrl) {
				this._updateTypeIcon(ImageClient.getImageUrl(this._workItem.attributes.workItemType.iconUrl));
			} else {
				this._updateTypeIcon(); /* hide it */
			}
			this._enableFindDuplicatesIcon();
		},
		
		//
		// 'public' methods available to the controller
		//
		
		disableSave: function(isDisabled) {
			isDisabled= isDisabled || this.parentController.workingCopy.isInOperation();
			this.saveCmd.disabled= isDisabled;
			this.parentController.disableSave(isDisabled);
			this._enableFindDuplicatesIcon();
			if (!isDisabled) {
				// Add the isDirty asterisk if it is not already there 
				if (this._titleTextNode.innerHTML && this._titleTextNode.innerHTML.indexOf("*") == -1) {
					this._titleTextNode.isDirty= true;
					this._titleTextNode.innerHTML= this._titleTextNode.innerHTML + " *"; /* innerHTML OK */
				}
			} else {
					this._titleTextNode.isDirty= false;
					this._titleTextNode.innerHTML= this._titleTextNode.innerHTML.replace(/ \*/, "");
			}
		},
				
		// implementation method for only the editor's form elements
		getFormElements: function() {
			return {};
		},
		
		markDirty: function() {
			// Enable top and bottom save button
			this.disableSave(/*isDisabled*/false);
		},
		
		/**
		 * Updates state of the "Subscribe Me" control to either add, remove or disabled
		 * based on what is currently allowed.
		 */
		updateSubscribeSelf: function() {
			if (!this._workItem) return;
			/*
			 * Keep the logged in contributor since this is expensive
			 */
			if(!this._loggedInContributor || this._loggedInContributor._isDeleted) {
				this._loggedInContributor = getAuthenticatedContributor();	
			}
			var loggedInContributor = this._loggedInContributor;
			var isEnabled= !this.parentController.isNewWorkItem(); 
			var isSubscriber= this.parentController.workingCopy.includesItem({
						item: loggedInContributor,
						attributeId: SUBSCRIBERS_ID
					});
			if (isEnabled) {
				if(!(this._subscribeSelfButtonEnabled === true)) {
					this._subscribeSelfButtonEnabled= true;
					this._subscribeSelfButton.set('enabled', true);					
				}
			}
			else {
				isSubscriber = false;
				if(this._subscribeSelfButtonEnabled === true) {
						this._subscribeSelfButtonEnabled= false;
						this._subscribeSelfButton.set('enabled', false);
					}
			}
			
			if (isEnabled && isSubscriber) {
				if(this._subscribeSelfButton.isAdd == null || this._subscribeSelfButton.isAdd === true) {
					this._subscribeSelfButton.isAdd= false;
					this._subscribeSelfButton.set('imgClass', Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, UNSUBSCRIBE)));
					this._subscribeSelfButton.set('title', this.titleUnsubscribeMe);
				}
			} else {
				if(this._subscribeSelfButton.isAdd == null || !(this._subscribeSelfButton.isAdd === true)) {
					this._subscribeSelfButton.isAdd= true;
					this._subscribeSelfButton.set('imgClass', Sprites.cssClassName(ImageSource.fromBundleImage(BUNDLE_ID, SUBSCRIBE)));
					this._subscribeSelfButton.set('title', this.titleSubscribeMe);
				}
			}
		},

		// update the type (e.g. defect or task) based on the type code (type change)
		updateType: function(name, imgSrc, isInitialization) {
			if (isInitialization && this._updatedTypeName) return;
			imgSrc= imgSrc ||ImagePool.PIXEL.toUri();
			if (!isInitialization) {
				this._updatedTypeName= name;
				this._updatedTypeIconSrc= imgSrc;
			}
			this.markBusy(this.messages.busyLoading);
		},
		
		_updateTypeIcon: function(imgSrc) {
			if (dojo.hasClass(this.typeIcon, "validationMessageErrorIcon")) {
				this.typeIcon.style.visibility= "visible";
				return;
			}

			if (!imgSrc) {
				this.typeIcon.style.visibility= "hidden";
			} else {			
				dojo.replaceClass(this.typeIcon, "TypeIcon " + Sprites.cssClassName(ImageSource.fromIconUri(imgSrc)));
				this.typeIcon.style.visibility= "visible";
			}
			this.typeIconSrc= imgSrc;
		},					
		
		//
		// ui initialization methods
		//
		
		_disableRefresh: function(isDisabled) {
			this._refreshButton.set('enabled', isDisabled ? false : true);
		},
		
		//
		// event handlers
		//
		// focus lost causes the propagation (e.g., to 
		_handleSummaryKeyUp: function() {
			this._checkSummaryValue();
		},		
		_handleSummaryBlur: function(e) {
			this._checkSummaryValue();
		},
		_checkSummaryValue: function(e) {
			var oldValue= this.parentController.workingCopy.getValue({path: "attributes.summary.content"});
			if (oldValue == this.getSummaryValue()) return;
			this._handleSummaryChange(e, /* notifyEvenIfUnchanged */ true);
		},
		// keystrokes update the value, but do not propagate
		_handleSummaryChange: function(e, notifyEvenIfUnchanged) {
			// parms can be null
			notifyEvenIfUnchanged= true; // always notify
			var isQuiet= this.parentController.workingCopy.isChanged() && (notifyEvenIfUnchanged === false);
			var value= this.getSummaryValue();
			if (this.parentController.workingCopy && this.parentController.workingCopy.object && this.parentController.workingCopy.object.attributes && this.parentController.workingCopy.object.attributes.summary) {
				if (value != this.parentController.workingCopy.object.attributes.summary.content) {
					this._setAttributeValue(this.getSummaryValue(), null, isQuiet, notifyEvenIfUnchanged);
					this.parentController.markDirty();
				}		
			}
		},
		_enableFindDuplicatesIcon: function() {
			if (!this._workItem) return;
			if (this._findDuplicatesButton.enabled) return; // already enabled
			var isEnabled= !this.parentController.workingCopy.isNewWorkItem();
			if (!isEnabled) {
				var val= this.getSummaryValue();
				if (val && val.length > 0) {
					isEnabled= true;
				}
				else {
					var descWidget= this.parentController.findObjectWithProperty({
						property: "isDescriptionManager"
					});
					if (descWidget && descWidget.getDescriptionValue) {
						if (descWidget.getDescriptionValue().length > 0) {
							isEnabled= true;
						}
					}
				}
			}
			
			this._findDuplicatesButton.set('enabled', isEnabled);
		},
 		_handleMoveCopyClick: function(e) {
			dojo.stopEvent(e);
			if (!this._moveCopyButton.enabled) return;
			this._openMoveCopyDialog();
		},
		displayMoveCopy: function(isEnabled) {
			this._moveCopyButton.set('visible', isEnabled);
			this._createWorkItemCopyAction.showAction(isEnabled);
		},
		_openMoveCopyDialog: function() {
			var self= this;
			var params= {
				projectArea: this.parentController.workingCopy.getValue({path: "attributes.projectArea"}),
				parentController: this.parentController,
				onOk: function(parms) {
					self._moveCopyOK(parms);
				},
				onClose: function() {
					self.parentController.focus();
				}
			};
			new com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog(params);

		},
		_moveCopyOK: function(parms /* isMove, projectAreaItemId */) {
			 var self= this;
				parms.onSuccess= function(args) {
					self._showInfosMessages(args);
				};
				parms.onError= function(args) {
					console.debug("move/copy error"); // $NON-NLS-1
					// failures have already been logged
					// TODO: test this path
				}
	    	this.clearValidationMessage(true);
			this.workingCopy.moveOrCopyToProjectArea(parms);
		},
		_showInfosMessages: function(parms) {
			this._currentInfos= parms.infos || this.parentController.workingCopy.infos;
			if (this._currentInfos && this._currentInfos.length !== 0) {
				this._showInfos(this._currentInfos);
			}
			if (parms.isMove) {
				this.parentController.markDirty();
			}
		},
		_handleFindDuplicatesClick: function(e) {
			dojo.stopEvent(e);
			if (!this._findDuplicatesButton.enabled) return;
			if (!this._workItem) return;
			if (this.parentController.workingCopy.isNewWorkItem()) { // new
				var summary= dojo.trim(this.getSummaryValue() || "");
				var content= ""; 
				
				var descWidget= this.parentController.findObjectWithProperty({
					property: "isDescriptionManager"
				});
				if (descWidget && descWidget.getDescriptionValue) {
					content=dojo.trim(descWidget.getDescriptionValue());
				}
				this.parentController.pageController.openSimilarityQuery( {
					summary: summary,
					content: content,
					projectAreaItemId: this.parentController.workingCopy.getProjectAreaItemId()
				});			
				
			} else {
				this.parentController.pageController.openSimilarityQuery( {
					itemId:  this._workItem.itemId,
					forceRefresh: true,
					projectAreaItemId: this.parentController.workingCopy.getProjectAreaItemId()
				});
			}			
		},
		_handleSubscribeSelfClick: function(e) {
			dojo.stopEvent(e);
			if (!this._subscribeSelfButton.enabled) return;
			if(!this._loggedInContributor || this._loggedInContributor._isDeleted) {
				this._loggedInContributor = getAuthenticatedContributor();	
			}
			var contributor = this._loggedInContributor;
			this.parentController.markDirty();
			if (this._subscribeSelfButton.isAdd) {
				this.parentController.workingCopy.addItems( {
					items: [contributor],
					attributeId: SUBSCRIBERS_ID
				});
			}
			else {
				this.parentController.workingCopy.deleteItems({
					items: [contributor],
					attributeId: SUBSCRIBERS_ID
				});
			}
			this.updateSubscribeSelf();
		},
		
		/*
		 * Respond to the user clicking the 'Save' control.
		 */
		_handleSaveClick: function(e) {
			this._lastSaveTime= new Date();
			this.parentController.save();
		},
		
		/*
		 * Respond to the user clicking the 'Refresh' control.
		 */
		_handleRefreshClick: function(e) {
			dojo.stopEvent(e);
			if (!this._refreshButton.enabled) return;
			// since we're going to update the work item
			this._lastSaveTime= null;
			this.parentController.refresh();
		},
		
		/*
		 * Respond to the user clicking the 'Cancel' button
		 */
		_handleCancelClick: function(e, skipConfirm) {
			if (e) dojo.stopEvent(e);
			if (!skipConfirm && !this.saveCmd.disabled) {
				if (!confirm(this.messages.promptCancelWorkItemEdit)) {
					return;
				}
			}
			this.parentController.pageController.removeEditor(this.parentController.workingCopy.getId(), true, true, this.workingCopy);
			this.parentController.destroy();
			this.parentController._sendChangedEvent("Closed", true);
		},
				
		clearValidationMessage: function(force) {
			if (!force && this._skipNextClear ) {
				this._skipNextClear= false;
				return;
			} 
			this._skipNextClear= false; // forced
			if (!this._hasStickyMessage) {
				this.validationMessage.__jazzMessage= -1;
				dojox.xml.parser.removeChildren(this.validationMessage);
			} 
			if (this.typeIconSrc) {
				this._updateTypeIcon(this.typeIconSrc);
			}
			dojo.removeClass(this.typeIcon, "validationMessageErrorIcon");
			this.validationMessageSpan.className= "validationMessageEmpty";
		
		},
		hasDuplicateLink: function() {
			 return this.parentController.hasDuplicateOfLink();
		},
		_showInfos: function(infos) {
			if (!infos || infos.length === 0 ) return;
				var msg;
			    if (infos.length > 1) {
			    	msg= dojo.string.substitute(this.messages.msgInfosMore, [ infos[0].summary, infos.length-1]);					
				} else {
					msg= infos[0].summary;
				}

				this.setHeaderMessage({
								message: msg,
								action: "SHOW_INFOS_ACTION",
								isSticky: false,
								isError: false
							});
		},
		setHeaderMessage: function(/* message, action, isSticky, severity */ message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
			dojo.addClass(this.typeIcon, "validationMessageErrorIcon");
			
			if (!message.message || message.message.length === 0) {
				message.message= this.messages.errorUnhelpful;
			} else {
				if (message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
            
            if (!message.details) {
                message.details= "";
            }
            else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText= message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            }
            else {
            	if (message.showOnlyMessage === true) {
					messageText= message.message;
				} else {
					messageText= message.message + " " + message.details;
				}
				if (messageText.length > MAX_MESSAGE_LENGTH) {
					messageText= messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
				}
			}
            messageText= messageText.replace(/\.\.\.\./, "...");
            
            this._hasStickyMessage= message.isSticky === true;
            this.validationMessage.__jazzMessage= message;
            if (message.severity == WorkItemClient.ERROR) {
				this._updateTypeIcon(ImagePool.ERROR.toUri());
                this.validationMessageSpan.className= "validationMessageError";
            }
            else {
				if (message.severity == WorkItemClient.WARNING) {
					this._updateTypeIcon(ImagePool.WARNING.toUri());
					this.validationMessageSpan.className= "validationMessageWarning";
					this._skipNextClear= true; // hack... we know there is an extra clear on setting resolutions
				} else {
					this._updateTypeIcon(ImagePool.INFO.toUri());
					this.validationMessageSpan.className= "validationMessageInfo";
					this._skipNextClear= true; // hack... we know there is an extra clear on setting resolutions					
				}
            }
            this.validationMessage.title= message.details || message.message;
            
            dojox.xml.parser.removeChildren(this.validationMessage);
            var textArray= messageText.split("\n");
            var messagesSpan= document.createElement("SPAN");
            for (var i= 0;i<textArray.length;i++) {
            	var div= document.createElement("DIV");
            	div.appendChild(document.createTextNode(textArray[i]));
            	messagesSpan.appendChild(div);
				messagesSpan.setAttribute("role", "alert");
            }
           	this.validationMessage.appendChild(messagesSpan);
			this.focus();
			window.scrollTo(0,0); // show message
		},
		updateValidationMessage: function(wasResolutionChanged) {
			// nothing to do now
		},
		_handleHeaderMessageClick: function(e) {
			dojo.stopEvent(e);
			if (!e) return;

			var message= this.validationMessage.__jazzMessage;
			if (!message) return;
			var actionFunc;
			if (dojo.isFunction(message.action)) {
				actionFunc= message.action;
			} else {				
				actionFunc= this.errorHandlingMap[message.action];
			}
			
			if (!actionFunc ) return;
			
			actionFunc(this,message );
 
		},
		/*
		 * Update the last saved or loaded timestamp
		 */
		updateRefreshTimestamp: function(isNew) {
			if (isNew) {
				this._refreshButton.set('title', this.messages.titleResetWorkItem);
				return;
			} else {
				this._refreshButton.set('title', this.messages.titleRefreshWorkItem);
			}
					
			var workItemProjectArea= this.parentController.workingCopy.getValue({path: "attributes.projectArea"});
			var currentProjectAreaItemId= com.ibm.team.workitem.web.ui.internal.module.WindowContext.getCurrentProjectArea().itemId;
			var now, prefix, timeString;
			if (!workItemProjectArea || workItemProjectArea.id == currentProjectAreaItemId) {
				if (this._lastSaveTime) {
					prefix= this.messages.labelSaved;
					now= this._lastSaveTime;
				} else {
					prefix= this.messages.labelLoaded;
					now= new Date();
				}
				var message;
				try {
					timeString= this.editorUtil._getDateString(now,true);
					message= dojo.string.substitute(prefix, [ timeString ]);
					
					dojox.xml.parser.replaceChildren(this.lastRefreshText, document.createTextNode(message));
	
					this.lastRefreshText.style.display="block";
	
				} catch(e) {
					// if there is a problem, just use the default now date
					this.lastRefreshText.style.display="none";
					console.debug(dojo.string.substitute(this.messages.errorInUpdateRefreshTimestamp, [ (""+e) , message ]));
				}
			} else {
				if (this._lastSaveTime) {
					prefix= this.messages.labelSavedToProjectArea;
					now= this._lastSaveTime;
				} else {
					prefix= this.messages.labelLoadedFromProjectArea;
					now= new Date();
				}
				var msg;
				try {
					timeString= this.editorUtil._getDateString(now, true);
					msg= dojo.string.substitute(prefix, [ timeString, workItemProjectArea.label ]);
					
					dojox.xml.parser.replaceChildren(this.lastRefreshText, document.createTextNode(msg));
	
					this.lastRefreshText.style.display="block";
	
				} catch(e) {
					// if there is a problem, just use the default now date
					this.lastRefreshText.style.display="none";
					console.debug(dojo.string.substitute(this.messages.errorInUpdateRefreshTimestamp, [ ("" + e) , msg ]));
				}			
			}
			
		},
				
		handleSaveComplete: function() {
			if (this.summary && this.summary.handleSaveComplete && dojo.isFunction(this.summary.handleSaveComplete)) {
				this.summary.handleSaveComplete();
			}
		}
	}
);

})();
